# autoflake: skip_file

from typing import TypeAlias, Union

from codetocad.core.angle import Angle
from codetocad.core.dimension import Dimension
from codetocad.core.dimensions import Dimensions
from codetocad.core.point import Point
from codetocad.core.boundary_box import BoundaryBox
from codetocad.core.boundary_axis import BoundaryAxis
from codetocad.core.part_options import PartOptions
from codetocad.core.sketch_options import SketchOptions
from codetocad.enums.angle_unit import AngleUnit
from codetocad.enums.axis import Axis
from codetocad.enums.constraint_types import ConstraintTypes
from codetocad.enums.curve_primitive_types import CurvePrimitiveTypes
from codetocad.enums.curve_types import CurveTypes
from codetocad.enums.length_unit import LengthUnit
from codetocad.enums.preset_landmark import PresetLandmark
from codetocad.enums.file_formats import FileFormats
from codetocad.enums.preset_material import PresetMaterial
from codetocad.enums.scaling_methods import ScalingMethods
from codetocad.enums.units import Units
from codetocad.enums.boolean_operation import BooleanOperation

# from codetocad.interfaces.booleanable_interface import BooleanableInterface
# from codetocad.interfaces.camera_interface import CameraInterface
# from codetocad.interfaces.entity_interface import EntityInterface
# from codetocad.interfaces.exportable_interface import ExportableInterface
# from codetocad.interfaces.landmark_interface import LandmarkInterface
# from codetocad.interfaces.landmarkable_interface import LandmarkableInterface
# from codetocad.interfaces.material_interface import MaterialInterface
# from codetocad.interfaces.part_interface import PartInterface
# from codetocad.interfaces.sketch_interface import SketchInterface
# from codetocad.interfaces.vertex_interface import VertexInterface



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from typing import Self


class ExportableInterface(metaclass=ABCMeta):
    """
    An enttiy that can be exported.
    """

    @abstractmethod
    def export(
        self, file_path: "str", overwrite: "bool" = True, scale: "float" = 1.0
    ) -> Self:
        """
        Export Entity. Use the filePath to control the export type, e.g. '/path/to/cube.obj' or '/path/to/curve.svg'
        """

        print("export is called in an abstract method. Please override this method.")

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


class MaterialInterface(metaclass=ABCMeta):
    """
    Materials affect the appearance and simulation properties of the parts.
    """

    @abstractmethod
    def __init__(self, name: "str", description: "str| None" = None):

        self.name = name
        self.description = description

    @staticmethod
    def get_preset(material_name: "PresetMaterial") -> "MaterialInterface":
        """
        Get a material from a preset
        """

        print(
            "get_preset is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_color(
        self,
        r_value: "int|float",
        g_value: "int|float",
        b_value: "int|float",
        a_value: "int|float" = 1.0,
    ) -> Self:
        """
        Set the RGBA color of an entity. Supports 0-255 int or 0.0-1.0 float values.
        """

        print("set_color is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def set_reflectivity(self, reflectivity: "float") -> Self:
        """
        Change the surface reflectivity (metallic luster) of the material.
        """

        print(
            "set_reflectivity is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_roughness(self, roughness: "float") -> Self:
        """
        Change the surface roughness of the material.
        """

        print(
            "set_roughness is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_image_texture(self, image_file_path: "str") -> Self:
        """
        Add a texture from an image file.
        """

        print(
            "set_image_texture is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from typing import Self


from codetocad.interfaces.entity_interface import EntityInterface


class CameraInterface(EntityInterface, metaclass=ABCMeta):
    """
    Manipulate a camera object.
    """

    @abstractmethod
    def __init__(
        self, name: "str", description: "str| None" = None, native_instance=None
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def create_perspective(
        self,
    ) -> Self:
        """
        Create a perspective camera in the scene.
        """

        print(
            "create_perspective is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_orthogonal(
        self,
    ) -> Self:
        """
        Create an orthogonal camera in the scene.
        """

        print(
            "create_orthogonal is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_panoramic(
        self,
    ) -> Self:
        """
        Create a panorama camera in the scene.
        """

        print(
            "create_panoramic is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_focal_length(self, length: "float") -> Self:
        """
        Set the focal length of the camera.
        """

        print(
            "set_focal_length is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



import unittest

from codetocad.enums.axis import Axis
from codetocad.enums.preset_landmark import PresetLandmark


class TestLandmarks(unittest.TestCase):
    def test_preset_landmarks(self):

        assert PresetLandmark.left.get_xyz() == (Axis.MIN, Axis.CENTER, Axis.CENTER)
        assert PresetLandmark.right.get_xyz() == (Axis.MAX, Axis.CENTER, Axis.CENTER)
        assert PresetLandmark.leftTop.get_xyz() == (Axis.MIN, Axis.CENTER, Axis.MAX)
        assert PresetLandmark.backTop.get_xyz() == (Axis.CENTER, Axis.MAX, Axis.MAX)
        assert PresetLandmark.rightBackBottom.get_xyz() == (
            Axis.MAX,
            Axis.MAX,
            Axis.MIN,
        )
        assert PresetLandmark.leftBackTop.get_xyz() == (Axis.MIN, Axis.MAX, Axis.MAX)
        assert PresetLandmark.center.get_xyz() == (
            Axis.CENTER,
            Axis.CENTER,
            Axis.CENTER,
        )



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.joint_interface import JointInterface


from codetocad.interfaces.entity_interface import EntityInterface


class Joint(
    JointInterface,
):

    def __init__(self, entity1: "str|EntityInterface", entity2: "str|EntityInterface"):

        self.entity1 = entity1
        self.entity2 = entity2

    def translate_landmark_onto_another(
        self,
    ) -> Self:

        print(
            "translate_landmark_onto_another called",
        )

        return self

    def pivot(
        self,
    ) -> Self:

        print(
            "pivot called",
        )

        return self

    def gear_ratio(self, ratio: "float") -> Self:

        print("gear_ratio called", f": {ratio}")

        return self

    def limit_location_xyz(
        self,
        x: "str|float|Dimension| None" = None,
        y: "str|float|Dimension| None" = None,
        z: "str|float|Dimension| None" = None,
    ) -> Self:

        print("limit_location_xyz called", f": {x}, {y}, {z}")

        return self

    def limit_location_x(
        self,
        min: "str|float|Dimension| None" = None,
        max: "str|float|Dimension| None" = None,
    ) -> Self:

        print("limit_location_x called", f": {min}, {max}")

        return self

    def limit_location_y(
        self,
        min: "str|float|Dimension| None" = None,
        max: "str|float|Dimension| None" = None,
    ) -> Self:

        print("limit_location_y called", f": {min}, {max}")

        return self

    def limit_location_z(
        self,
        min: "str|float|Dimension| None" = None,
        max: "str|float|Dimension| None" = None,
    ) -> Self:

        print("limit_location_z called", f": {min}, {max}")

        return self

    def limit_rotation_xyz(
        self,
        x: "str|float|Angle| None" = None,
        y: "str|float|Angle| None" = None,
        z: "str|float|Angle| None" = None,
    ) -> Self:

        print("limit_rotation_xyz called", f": {x}, {y}, {z}")

        return self

    def limit_rotation_x(
        self, min: "str|float|Angle| None" = None, max: "str|float|Angle| None" = None
    ) -> Self:

        print("limit_rotation_x called", f": {min}, {max}")

        return self

    def limit_rotation_y(
        self, min: "str|float|Angle| None" = None, max: "str|float|Angle| None" = None
    ) -> Self:

        print("limit_rotation_y called", f": {min}, {max}")

        return self

    def limit_rotation_z(
        self, min: "str|float|Angle| None" = None, max: "str|float|Angle| None" = None
    ) -> Self:

        print("limit_rotation_z called", f": {min}, {max}")

        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.landmark_interface import LandmarkInterface

from codetocad.interfaces.material_interface import MaterialInterface

from codetocad.interfaces.importable_interface import ImportableInterface

from codetocad.interfaces.exportable_interface import ExportableInterface

from codetocad.interfaces.subdividable_interface import SubdividableInterface

from codetocad.interfaces.scalable_interface import ScalableInterface

from codetocad.interfaces.landmarkable_interface import LandmarkableInterface

from codetocad.interfaces.mirrorable_interface import MirrorableInterface

from codetocad.interfaces.booleanable_interface import BooleanableInterface

from codetocad.interfaces.patternable_interface import PatternableInterface

from codetocad.interfaces.entity_interface import EntityInterface


class PartInterface(
    EntityInterface,
    MirrorableInterface,
    PatternableInterface,
    SubdividableInterface,
    ImportableInterface,
    ExportableInterface,
    ScalableInterface,
    LandmarkableInterface,
    BooleanableInterface,
    metaclass=ABCMeta,
):
    """
    Capabilities related to creating and manipulating 3D shapes.
    """

    @abstractmethod
    def __init__(
        self, name: "str", description: "str| None" = None, native_instance=None
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def create_cube(
        self,
        width: "str|float|Dimension",
        length: "str|float|Dimension",
        height: "str|float|Dimension",
        options: "PartOptions| None" = None,
    ) -> Self:
        """
        Adds cuboid geometry to a part.
        """

        print(
            "create_cube is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_cone(
        self,
        radius: "str|float|Dimension",
        height: "str|float|Dimension",
        draft_radius: "str|float|Dimension" = 0,
        options: "PartOptions| None" = None,
    ) -> Self:
        """
        Adds cone geometry to a part.
        """

        print(
            "create_cone is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_cylinder(
        self,
        radius: "str|float|Dimension",
        height: "str|float|Dimension",
        options: "PartOptions| None" = None,
    ) -> Self:
        """
        Adds cylinder geometry to a part.
        """

        print(
            "create_cylinder is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_torus(
        self,
        inner_radius: "str|float|Dimension",
        outer_radius: "str|float|Dimension",
        options: "PartOptions| None" = None,
    ) -> Self:
        """
        Adds torus geometry to a part.
        """

        print(
            "create_torus is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_sphere(
        self, radius: "str|float|Dimension", options: "PartOptions| None" = None
    ) -> Self:
        """
        Adds sphere geometry to a part.
        """

        print(
            "create_sphere is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_gear(
        self,
        outer_radius: "str|float|Dimension",
        addendum: "str|float|Dimension",
        inner_radius: "str|float|Dimension",
        dedendum: "str|float|Dimension",
        height: "str|float|Dimension",
        pressure_angle: "str|float|Angle" = "20d",
        number_of_teeth: "int" = 12,
        skew_angle: "str|float|Angle" = 0,
        conical_angle: "str|float|Angle" = 0,
        crown_angle: "str|float|Angle" = 0,
        options: "PartOptions| None" = None,
    ) -> Self:
        """
        Adds gear geometry to a part.
        """

        print(
            "create_gear is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def clone(self, new_name: "str", copy_landmarks: "bool" = True) -> "PartInterface":
        """
        Clone an existing Part with its geometry and properties. Returns the new Part.
        """

        print("clone is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def hollow(
        self,
        thickness_x: "str|float|Dimension",
        thickness_y: "str|float|Dimension",
        thickness_z: "str|float|Dimension",
        start_axis: "str|int|Axis" = "z",
        flip_axis: "bool" = False,
    ) -> Self:
        """
        Remove vertices, if necessary, until the part has a specified wall thickness.
        """

        print("hollow is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def thicken(self, radius: "str|float|Dimension") -> Self:
        """
        Uniformly add a wall around a Part.
        """

        print("thicken is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def hole(
        self,
        hole_landmark: "str|LandmarkInterface",
        radius: "str|float|Dimension",
        depth: "str|float|Dimension",
        normal_axis: "str|int|Axis" = "z",
        flip_axis: "bool" = False,
        initial_rotation_x: "str|float|Angle" = 0.0,
        initial_rotation_y: "str|float|Angle" = 0.0,
        initial_rotation_z: "str|float|Angle" = 0.0,
        mirror_about_entity_or_landmark: "str|EntityInterface| None" = None,
        mirror_axis: "str|int|Axis" = "x",
        mirror: "bool" = False,
        circular_pattern_instance_count: "int" = 1,
        circular_pattern_instance_separation: "str|float|Angle" = 0.0,
        circular_pattern_instance_axis: "str|int|Axis" = "z",
        circular_pattern_about_entity_or_landmark: "str|EntityInterface| None" = None,
        linear_pattern_instance_count: "int" = 1,
        linear_pattern_instance_separation: "str|float|Dimension" = 0.0,
        linear_pattern_instance_axis: "str|int|Axis" = "x",
        linear_pattern2nd_instance_count: "int" = 1,
        linear_pattern2nd_instance_separation: "str|float|Dimension" = 0.0,
        linear_pattern2nd_instance_axis: "str|int|Axis" = "y",
    ) -> Self:
        """
        Create a hole.
        """

        print("hole is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def twist(
        self,
        angle: "str|float|Angle",
        screw_pitch: "str|float|Dimension",
        iterations: "int" = 1,
        axis: "str|int|Axis" = "z",
    ) -> Self:
        """
        AKA Helix, Screw. Revolve an entity
        """

        print("twist is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def set_material(self, material_name: "str|MaterialInterface") -> Self:
        """
        Assign a known material to this part.
        """

        print(
            "set_material is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def is_colliding_with_part(self, other_part: "str|PartInterface") -> "bool":
        """
        Check if this part is colliding with another.
        """

        print(
            "is_colliding_with_part is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def fillet_all_edges(
        self, radius: "str|float|Dimension", use_width: "bool" = False
    ) -> Self:
        """
        Fillet all edges.
        """

        print(
            "fillet_all_edges is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def fillet_edges(
        self,
        radius: "str|float|Dimension",
        landmarks_near_edges: "list[str|LandmarkInterface]",
        use_width: "bool" = False,
    ) -> Self:
        """
        Fillet specific edges.
        """

        print(
            "fillet_edges is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def fillet_faces(
        self,
        radius: "str|float|Dimension",
        landmarks_near_faces: "list[str|LandmarkInterface]",
        use_width: "bool" = False,
    ) -> Self:
        """
        Fillet specific faces.
        """

        print(
            "fillet_faces is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def chamfer_all_edges(self, radius: "str|float|Dimension") -> Self:
        """
        Chamfer all edges.
        """

        print(
            "chamfer_all_edges is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def chamfer_edges(
        self,
        radius: "str|float|Dimension",
        landmarks_near_edges: "list[str|LandmarkInterface]",
    ) -> Self:
        """
        Chamfer specific edges.
        """

        print(
            "chamfer_edges is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def chamfer_faces(
        self,
        radius: "str|float|Dimension",
        landmarks_near_faces: "list[str|LandmarkInterface]",
    ) -> Self:
        """
        Chamfer specific faces.
        """

        print(
            "chamfer_faces is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def select_vertex_near_landmark(
        self, landmark_name: "str|LandmarkInterface| None" = None
    ) -> Self:
        """
        Select the vertex closest to a Landmark on the entity (in UI).
        """

        print(
            "select_vertex_near_landmark is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def select_edge_near_landmark(
        self, landmark_name: "str|LandmarkInterface| None" = None
    ) -> Self:
        """
        Select an edge closest to a landmark on the entity (in UI).
        """

        print(
            "select_edge_near_landmark is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def select_face_near_landmark(
        self, landmark_name: "str|LandmarkInterface| None" = None
    ) -> Self:
        """
        Select a face closest to a landmark on the entity (in UI).
        """

        print(
            "select_face_near_landmark is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



from dataclasses import dataclass

from codetocad.core.angle import Angle
from codetocad.enums.boolean_operation import BooleanOperation
from codetocad.enums.preset_landmark import PresetLandmark
from codetocad.proxy.landmark import Landmark


@dataclass
class PartOptions:
    """
    :param merge_at: The landmark where this shape will be inserted at. default_value: PresetLandmark.bottom
    :param merge_with: The landmark where this shape will be inserted at relative to existing geometry in the Part. default_value: PresetLandmark.top
    :param rotation_x: If a rotation is not specified, the shape will snap to the normal of the merge_with face.
    :param rotation_y: If a rotation is not specified, the shape will snap to the normal of the merge_with face
    :param rotation_z: If a rotation is not specified, the shape will snap to the normal of the merge_with face
    :param boolean_operation: "Union merge by default. Available options: union, subtract, intersect. default_value: BooleanOperation.UNION
    """

    merge_at: str | Landmark | PresetLandmark = PresetLandmark.bottom
    merge_with: str | Landmark | PresetLandmark = PresetLandmark.top
    rotation_x: str | float | Angle | None = None
    rotation_y: str | float | Angle | None = None
    rotation_z: str | float | Angle | None = None
    boolean_operation: str | BooleanOperation = BooleanOperation.UNION



import os
import subprocess
from codetocad.launchers.launcher_args import LauncherArgs


def build_blender_subprocess_args(launcher_args: LauncherArgs):
    """
    builds a path like:
    blender myscene.blend --background -- --codetocad $(pwd)/yourScript.py

    from LauncherArgs
    """

    args = [launcher_args.launcher_location or launcher_args.launcher]

    if launcher_args.document_name:
        # todo: check for the .blend suffix
        args.append(launcher_args.document_name)

    if launcher_args.background:
        args.append("--background")

    args += ["--", "--codetocad", launcher_args.script_file_path]

    return args


def run_blender_process(launcher_args: LauncherArgs):
    """
    Attempts to launch blender and run the provided script path.
    The CodeToCAD Blender Addon must be installed for this to work.
    """
    return subprocess.Popen(
        build_blender_subprocess_args(launcher_args), env=os.environ
    )



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


class ScalableInterface(metaclass=ABCMeta):
    """
    An entity that can be transformed by a scale.
    """

    @abstractmethod
    def scale_xyz(
        self,
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> Self:
        """
        Scale in the XYZ directions. Pass a number, Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print("scale_xyz is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def scale_x(self, scale: "str|float|Dimension") -> Self:
        """
        Scale in the X direction. Pass a number, Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print("scale_x is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def scale_y(self, scale: "str|float|Dimension") -> Self:
        """
        Scale in the Y direction. Pass a number, Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print("scale_y is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def scale_z(self, scale: "str|float|Dimension") -> Self:
        """
        Scale in the Z direction. Pass a number, Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print("scale_z is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def scale_x_by_factor(self, scale_factor: "float") -> Self:
        """
        Scale in the X direction by a multiple.
        """

        print(
            "scale_x_by_factor is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def scale_y_by_factor(self, scale_factor: "float") -> Self:
        """
        Scale in the Y direction by a multiple.
        """

        print(
            "scale_y_by_factor is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def scale_z_by_factor(self, scale_factor: "float") -> Self:
        """
        Scale in the X direction by a multiple.
        """

        print(
            "scale_z_by_factor is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def scale_keep_aspect_ratio(
        self, scale: "str|float|Dimension", axis: "str|int|Axis"
    ) -> Self:
        """
        Scale in one axis and maintain the others. Pass a Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print(
            "scale_keep_aspect_ratio is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.wire_interface import WireInterface


from codetocad.interfaces.vertex_interface import VertexInterface

from codetocad.interfaces.part_interface import PartInterface

from codetocad.interfaces.landmark_interface import LandmarkInterface

from codetocad.interfaces.edge_interface import EdgeInterface


from codetocad.interfaces.booleanable_interface import BooleanableInterface

from codetocad.interfaces.projectable_interface import ProjectableInterface


from codetocad.interfaces.entity_interface import EntityInterface


from codetocad.proxy.vertex import Vertex

from codetocad.proxy.part import Part

from codetocad.proxy.landmark import Landmark

from codetocad.proxy.edge import Edge


from providers.sample.entity import Entity


class Wire(WireInterface, Entity):

    def __init__(
        self,
        name: "str",
        edges: "list[EdgeInterface]",
        description: "str| None" = None,
        native_instance=None,
        parent_entity: "str|EntityInterface| None" = None,
    ):

        self.name = name
        self.edges = edges
        self.description = description
        self.native_instance = native_instance
        self.parent_entity = parent_entity

    def get_normal(self, flip: "bool| None" = False) -> "Point":

        print("get_normal called", f": {flip}")

        return Point.from_list_of_float_or_string([0, 0, 0])

    def get_edges(
        self,
    ) -> "list[EdgeInterface]":

        print(
            "get_edges called",
        )

        return [
            Edge(
                v1=Vertex("a vertex", Point.from_list_of_float_or_string([0, 0, 0])),
                v2=Vertex("a vertex", Point.from_list_of_float_or_string([0, 0, 0])),
                name="an edge",
            )
        ]

    def get_vertices(
        self,
    ) -> "list[VertexInterface]":

        print(
            "get_vertices called",
        )

        return [Vertex("a vertex", Point.from_list_of_float_or_string([0, 0, 0]))]

    def get_is_closed(
        self,
    ) -> "bool":

        print(
            "get_is_closed called",
        )

        return True

    def loft(
        self, other: "WireInterface", new_part_name: "str| None" = None
    ) -> "PartInterface":

        print("loft called", f": {other}, {new_part_name}")

        return Part("a part")

    def revolve(
        self,
        angle: "str|float|Angle",
        about_entity_or_landmark: "str|EntityInterface",
        axis: "str|int|Axis" = "z",
    ) -> "PartInterface":

        print("revolve called", f": {angle}, {about_entity_or_landmark}, {axis}")

        return Part("a part")

    def twist(
        self,
        angle: "str|float|Angle",
        screw_pitch: "str|float|Dimension",
        iterations: "int" = 1,
        axis: "str|int|Axis" = "z",
    ) -> Self:

        print("twist called", f": {angle}, {screw_pitch}, {iterations}, {axis}")

        return self

    def extrude(self, length: "str|float|Dimension") -> "PartInterface":

        print("extrude called", f": {length}")

        return Part("a part")

    def sweep(
        self, profile_name_or_instance: "str|WireInterface", fill_cap: "bool" = True
    ) -> "PartInterface":

        print("sweep called", f": {profile_name_or_instance}, {fill_cap}")

        return Part("a part")

    def offset(self, radius: "str|float|Dimension") -> "WireInterface":

        print("offset called", f": {radius}")

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def profile(self, profile_curve_name: "str") -> Self:

        print("profile called", f": {profile_curve_name}")

        return self

    def create_from_vertices(
        self,
        points: "list[str|list[str]|list[float]|list[Dimension]|Point|VertexInterface]",
        options: "SketchOptions| None" = None,
    ) -> Self:

        print("create_from_vertices called", f": {points}, {options}")

        return self

    def create_point(
        self,
        point: "str|list[str]|list[float]|list[Dimension]|Point",
        options: "SketchOptions| None" = None,
    ) -> Self:

        print("create_point called", f": {point}, {options}")

        return self

    def create_line(
        self,
        length: "str|float|Dimension",
        angle: "str|float|Angle",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        options: "SketchOptions| None" = None,
    ) -> Self:

        print("create_line called", f": {length}, {angle}, {start_at}, {options}")

        return self

    def create_line_to(
        self,
        to: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        options: "SketchOptions| None" = None,
    ) -> Self:

        print("create_line_to called", f": {to}, {start_at}, {options}")

        return self

    def create_arc(
        self,
        end_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface",
        radius: "str|float|Dimension",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        flip: "bool| None" = False,
        options: "SketchOptions| None" = None,
    ) -> Self:

        print(
            "create_arc called", f": {end_at}, {radius}, {start_at}, {flip}, {options}"
        )

        return self

    def mirror(
        self,
        mirror_across_entity: "str|EntityInterface",
        axis: "str|int|Axis",
        resulting_mirrored_entity_name: "str| None" = None,
    ) -> Self:

        print(
            "mirror called",
            f": {mirror_across_entity}, {axis}, {resulting_mirrored_entity_name}",
        )

        return self

    def linear_pattern(
        self,
        instance_count: "int",
        offset: "str|float|Dimension",
        direction_axis: "str|int|Axis" = "z",
    ) -> Self:

        print(
            "linear_pattern called", f": {instance_count}, {offset}, {direction_axis}"
        )

        return self

    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: "str|float|Angle",
        center_entity_or_landmark: "str|EntityInterface",
        normal_direction_axis: "str|int|Axis" = "z",
    ) -> Self:

        print(
            "circular_pattern called",
            f": {instance_count}, {separation_angle}, {center_entity_or_landmark}, {normal_direction_axis}",
        )

        return self

    def project(self, project_from: "ProjectableInterface") -> "ProjectableInterface":

        print("project called", f": {project_from}")

        return __import__("codetocad").Sketch("a projected sketch")

    def create_landmark(
        self,
        landmark_name: "str",
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> "LandmarkInterface":

        print("create_landmark called", f": {landmark_name}, {x}, {y}, {z}")

        return Landmark("name", "parent")

    def get_landmark(self, landmark_name: "str|PresetLandmark") -> "LandmarkInterface":

        print("get_landmark called", f": {landmark_name}")

        return Landmark("name", "parent")

    def union(
        self,
        other: "str|BooleanableInterface",
        delete_after_union: "bool" = True,
        is_transfer_data: "bool" = False,
    ) -> Self:

        print("union called", f": {other}, {delete_after_union}, {is_transfer_data}")

        return self

    def subtract(
        self,
        other: "str|BooleanableInterface",
        delete_after_subtract: "bool" = True,
        is_transfer_data: "bool" = False,
    ) -> Self:

        print(
            "subtract called", f": {other}, {delete_after_subtract}, {is_transfer_data}"
        )

        return self

    def intersect(
        self,
        other: "str|BooleanableInterface",
        delete_after_intersect: "bool" = True,
        is_transfer_data: "bool" = False,
    ) -> Self:

        print(
            "intersect called",
            f": {other}, {delete_after_intersect}, {is_transfer_data}",
        )

        return self

    def remesh(self, strategy: "str", amount: "float") -> Self:

        print("remesh called", f": {strategy}, {amount}")

        return self

    def subdivide(self, amount: "float") -> Self:

        print("subdivide called", f": {amount}")

        return self

    def decimate(self, amount: "float") -> Self:

        print("decimate called", f": {amount}")

        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.entity_interface import EntityInterface


class MirrorableInterface(metaclass=ABCMeta):
    """
    An entity that can be mirrored.
    """

    @abstractmethod
    def mirror(
        self,
        mirror_across_entity: "str|EntityInterface",
        axis: "str|int|Axis",
        resulting_mirrored_entity_name: "str| None" = None,
    ) -> Self:
        """
        Mirror an existing entity with respect to a landmark. If a name is provided, the mirror becomes a separate entity.
        """

        print("mirror is called in an abstract method. Please override this method.")

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *


from codetocad.interfaces.landmark_interface import LandmarkInterface


class LandmarkableInterface(metaclass=ABCMeta):
    """
    An entity that can be use landmarks.
    """

    @abstractmethod
    def create_landmark(
        self,
        landmark_name: "str",
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> "LandmarkInterface":
        """
        Shortcut for creating and assigning a landmark to this entity. Returns a Landmark instance.
        """

        print(
            "create_landmark is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_landmark(self, landmark_name: "str|PresetLandmark") -> "LandmarkInterface":
        """
        Get the landmark by name
        """

        print(
            "get_landmark is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from typing import Self


class ImportableInterface(metaclass=ABCMeta):
    """
    An entity that can be imported from a file.
    """

    @abstractmethod
    def create_from_file(self, file_path: "str", file_type: "str| None" = None) -> Self:
        """
        Imports geometry from a file.
        """

        print(
            "create_from_file is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.sketch_interface import SketchInterface


from codetocad.interfaces.vertex_interface import VertexInterface

from codetocad.interfaces.wire_interface import WireInterface

from codetocad.interfaces.landmark_interface import LandmarkInterface


from codetocad.interfaces.projectable_interface import ProjectableInterface


from codetocad.interfaces.entity_interface import EntityInterface


from codetocad.proxy.edge import Edge
from codetocad.proxy.vertex import Vertex

from codetocad.proxy.wire import Wire

from codetocad.proxy.landmark import Landmark


from providers.sample.entity import Entity


class Sketch(SketchInterface, Entity):

    def __init__(
        self,
        name: "str",
        description: "str| None" = None,
        native_instance=None,
        curve_type: "CurveTypes| None" = None,
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance
        self.curve_type = curve_type

    def get_wires(
        self,
    ) -> "list[WireInterface]":

        print(
            "get_wires called",
        )

        return [
            Wire(
                "a wire",
                [
                    Edge(
                        v1=Vertex(
                            "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                        ),
                        v2=Vertex(
                            "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                        ),
                        name="an edge",
                    )
                ],
            )
        ]

    def clone(
        self, new_name: "str", copy_landmarks: "bool" = True
    ) -> "SketchInterface":

        print("clone called", f": {new_name}, {copy_landmarks}")

        return Sketch("a sketch")

    def create_text(
        self,
        text: "str",
        font_size: "str|float|Dimension" = 1.0,
        bold: "bool" = False,
        italic: "bool" = False,
        underlined: "bool" = False,
        character_spacing: "int" = 1,
        word_spacing: "int" = 1,
        line_spacing: "int" = 1,
        font_file_path: "str| None" = None,
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print(
            "create_text called",
            f": {text}, {font_size}, {bold}, {italic}, {underlined}, {character_spacing}, {word_spacing}, {line_spacing}, {font_file_path}, {center_at}, {options}",
        )

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_from_vertices(
        self,
        points: "list[str|list[str]|list[float]|list[Dimension]|Point|VertexInterface]",
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print("create_from_vertices called", f": {points}, {options}")

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_point(
        self,
        point: "str|list[str]|list[float]|list[Dimension]|Point",
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print("create_point called", f": {point}, {options}")

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_line(
        self,
        length: "str|float|Dimension",
        angle: "str|float|Angle",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print("create_line called", f": {length}, {angle}, {start_at}, {options}")

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_line_to(
        self,
        to: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print("create_line_to called", f": {to}, {start_at}, {options}")

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_circle(
        self,
        radius: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print("create_circle called", f": {radius}, {center_at}, {options}")

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_ellipse(
        self,
        radius_minor: "str|float|Dimension",
        radius_major: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print(
            "create_ellipse called",
            f": {radius_minor}, {radius_major}, {center_at}, {options}",
        )

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_arc(
        self,
        end_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface",
        radius: "str|float|Dimension",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        flip: "bool| None" = False,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print(
            "create_arc called", f": {end_at}, {radius}, {start_at}, {flip}, {options}"
        )

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_rectangle(
        self,
        length: "str|float|Dimension",
        width: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print("create_rectangle called", f": {length}, {width}, {center_at}, {options}")

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_polygon(
        self,
        number_of_sides: "int",
        length: "str|float|Dimension",
        width: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print(
            "create_polygon called",
            f": {number_of_sides}, {length}, {width}, {center_at}, {options}",
        )

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_trapezoid(
        self,
        length_upper: "str|float|Dimension",
        length_lower: "str|float|Dimension",
        height: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print(
            "create_trapezoid called",
            f": {length_upper}, {length_lower}, {height}, {center_at}, {options}",
        )

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def create_spiral(
        self,
        number_of_turns: "int",
        height: "str|float|Dimension",
        radius: "str|float|Dimension",
        is_clockwise: "bool" = True,
        radius_end: "str|float|Dimension| None" = None,
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":

        print(
            "create_spiral called",
            f": {number_of_turns}, {height}, {radius}, {is_clockwise}, {radius_end}, {center_at}, {options}",
        )

        return Wire(
            "a wire",
            [
                Edge(
                    v1=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    v2=Vertex(
                        "a vertex", Point.from_list_of_float_or_string([0, 0, 0])
                    ),
                    name="an edge",
                )
            ],
        )

    def mirror(
        self,
        mirror_across_entity: "str|EntityInterface",
        axis: "str|int|Axis",
        resulting_mirrored_entity_name: "str| None" = None,
    ) -> Self:

        print(
            "mirror called",
            f": {mirror_across_entity}, {axis}, {resulting_mirrored_entity_name}",
        )

        return self

    def linear_pattern(
        self,
        instance_count: "int",
        offset: "str|float|Dimension",
        direction_axis: "str|int|Axis" = "z",
    ) -> Self:

        print(
            "linear_pattern called", f": {instance_count}, {offset}, {direction_axis}"
        )

        return self

    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: "str|float|Angle",
        center_entity_or_landmark: "str|EntityInterface",
        normal_direction_axis: "str|int|Axis" = "z",
    ) -> Self:

        print(
            "circular_pattern called",
            f": {instance_count}, {separation_angle}, {center_entity_or_landmark}, {normal_direction_axis}",
        )

        return self

    def create_from_file(self, file_path: "str", file_type: "str| None" = None) -> Self:

        print("create_from_file called", f": {file_path}, {file_type}")

        return self

    def export(
        self, file_path: "str", overwrite: "bool" = True, scale: "float" = 1.0
    ) -> Self:

        print("export called", f": {file_path}, {overwrite}, {scale}")

        return self

    def scale_xyz(
        self,
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> Self:

        print("scale_xyz called", f": {x}, {y}, {z}")

        return self

    def scale_x(self, scale: "str|float|Dimension") -> Self:

        print("scale_x called", f": {scale}")

        return self

    def scale_y(self, scale: "str|float|Dimension") -> Self:

        print("scale_y called", f": {scale}")

        return self

    def scale_z(self, scale: "str|float|Dimension") -> Self:

        print("scale_z called", f": {scale}")

        return self

    def scale_x_by_factor(self, scale_factor: "float") -> Self:

        print("scale_x_by_factor called", f": {scale_factor}")

        return self

    def scale_y_by_factor(self, scale_factor: "float") -> Self:

        print("scale_y_by_factor called", f": {scale_factor}")

        return self

    def scale_z_by_factor(self, scale_factor: "float") -> Self:

        print("scale_z_by_factor called", f": {scale_factor}")

        return self

    def scale_keep_aspect_ratio(
        self, scale: "str|float|Dimension", axis: "str|int|Axis"
    ) -> Self:

        print("scale_keep_aspect_ratio called", f": {scale}, {axis}")

        return self

    def project(self, project_from: "ProjectableInterface") -> "ProjectableInterface":

        print("project called", f": {project_from}")

        return __import__("codetocad").Sketch("a projected sketch")

    def create_landmark(
        self,
        landmark_name: "str",
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> "LandmarkInterface":

        print("create_landmark called", f": {landmark_name}, {x}, {y}, {z}")

        return Landmark("name", "parent")

    def get_landmark(self, landmark_name: "str|PresetLandmark") -> "LandmarkInterface":

        print("get_landmark called", f": {landmark_name}")

        return Landmark("name", "parent")



import unittest

from development.capabilities_json_to_python.capabilities_loader import (
    CapabilitiesLoader,
)


class TestTemplating(unittest.TestCase):
    capabilies_loader = CapabilitiesLoader()

    def test_generate_methods(self):
        imports = self.capabilies_loader.generate_imports("Sketch", ["Sketch"])

        assert imports._codetocad_implementable_class_names == {
            "Landmark",
            "Wire",
            "Vertex",
        }
        assert imports._codetocad_interface_class_names == {
            "Projectable",
            "Importable",
            "Mirrorable",
            "Landmarkable",
            "Exportable",
            "Scalable",
            "Patternable",
        }

        imports = self.capabilies_loader.generate_imports("Part", ["Part"])

        assert imports._codetocad_implementable_class_names == {
            "Landmark",
            "Material",
        }

        assert imports._codetocad_interface_class_names == {
            "Exportable",
            "Booleanable",
            "Subdividable",
            "Scalable",
            "Importable",
            "Landmarkable",
            "Mirrorable",
            "Patternable",
        }

        imports = self.capabilies_loader.generate_imports("Edge", ["Edge"])

        assert imports._codetocad_implementable_class_names == {"Vertex", "Landmark"}
        assert imports._codetocad_interface_class_names == {
            "Projectable",
            "Patternable",
            "Subdividable",
            "Landmarkable",
            "Mirrorable",
        }

        imports = self.capabilies_loader.generate_imports("Wire", ["Wire"])

        assert imports._codetocad_implementable_class_names == {
            "Vertex",
            "Landmark",
            "Edge",
            "Part",
        }

        assert imports._codetocad_interface_class_names == {
            "Booleanable",
            "Landmarkable",
            "Patternable",
            "Mirrorable",
            "Projectable",
            "Subdividable",
        }

    def test_init_parameters(self):
        parameters = self.capabilies_loader.get_constructor_parameters_for_class(
            "Vertex"
        )

        has_default_value = False

        for parameter in parameters:
            if parameter.default_value is None and parameter.required:
                assert (
                    has_default_value is False
                ), "Argument with non-default value found after one with a default argument"
            else:
                has_default_value = True



from .angle_unit import AngleUnit
from .axis import Axis
from .constraint_types import ConstraintTypes
from .curve_primitive_types import CurvePrimitiveTypes
from .curve_types import CurveTypes
from .file_formats import FileFormats
from .length_unit import LengthUnit
from .preset_landmark import PresetLandmark
from .preset_material import PresetMaterial
from .scaling_methods import ScalingMethods
from .units import Units
from .boolean_operation import BooleanOperation



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *


from codetocad.interfaces.entity_interface import EntityInterface


class LandmarkInterface(EntityInterface, metaclass=ABCMeta):
    """
    Landmarks are named positions on an entity.
    """

    @abstractmethod
    def __init__(
        self,
        name: "str",
        parent_entity: "str|EntityInterface",
        description: "str| None" = None,
        native_instance=None,
    ):

        self.name = name
        self.parent_entity = parent_entity
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def clone(
        self,
        new_name: "str",
        offset: "str|list[str]|list[float]|list[Dimension]|Dimensions| None" = None,
        new_parent: "str|EntityInterface| None" = None,
    ) -> "LandmarkInterface":
        """
        Clone an existing Landmark with an optional offset, and reassignment to a different parent. Returns the new Landmark.
        """

        print("clone is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def get_landmark_entity_name(
        self,
    ) -> "str":
        """
        Get the landmark object name in the scene, which may be different to the name of the landmark when it was first created. For example, the generated name may be {parentName}_{landmarkName}.
        """

        print(
            "get_landmark_entity_name is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_parent_entity(
        self,
    ) -> "EntityInterface":
        """
        Get the name of the entity this landmark belongs to.
        """

        print(
            "get_parent_entity is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from typing import Self


from codetocad.interfaces.camera_interface import CameraInterface


class RenderInterface(metaclass=ABCMeta):
    """
    Render the scene and export images or videos.
    """

    @abstractmethod
    def render_image(
        self,
        output_file_path: "str",
        overwrite: "bool" = True,
        file_type: "str| None" = None,
    ) -> Self:
        """
        Render a still image.
        """

        print(
            "render_image is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def render_video_mp4(
        self,
        output_file_path: "str",
        start_frame_number: "int" = 1,
        end_frame_number: "int" = 100,
        step_frames: "int" = 1,
        overwrite: "bool" = True,
    ) -> Self:
        """
        Render an MP4 video.
        """

        print(
            "render_video_mp4 is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def render_video_frames(
        self,
        output_folder_path: "str",
        file_name_prefix: "str",
        start_frame_number: "int" = 1,
        end_frame_number: "int" = 100,
        step_frames: "int" = 1,
        overwrite: "bool" = True,
        file_type: "str| None" = None,
    ) -> Self:
        """
        Render a video as image frame stills.
        """

        print(
            "render_video_frames is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_frame_rate(self, frame_rate: "int") -> Self:
        """
        Set rendering framerate.
        """

        print(
            "set_frame_rate is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_resolution(self, x: "int", y: "int") -> Self:
        """
        Set rendering resolution
        """

        print(
            "set_resolution is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_render_quality(self, quality: "int") -> Self:
        """
        Set rendering quality.
        """

        print(
            "set_render_quality is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_render_engine(self, name: "str") -> Self:
        """
        Set rendering engine name.
        """

        print(
            "set_render_engine is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_camera(self, camera_name_or_instance: "str|CameraInterface") -> Self:
        """
        Set the rendering camera.
        """

        print(
            "set_camera is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from typing import Self


from codetocad.interfaces.entity_interface import EntityInterface


class AnimationInterface(metaclass=ABCMeta):
    """
    Animation related functionality.
    """

    @staticmethod
    def default() -> "AnimationInterface":
        """
        Get an Animation instance for the current scene.
        """

        print("default is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def set_frame_start(self, frame_number: "int") -> Self:
        """
        Set the start animation frame in the scene.
        """

        print(
            "set_frame_start is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_frame_end(self, frame_number: "int") -> Self:
        """
        Set the end animation frame in the scene.
        """

        print(
            "set_frame_end is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_frame_current(self, frame_number: "int") -> Self:
        """
        Set the current animation frame in the scene.
        """

        print(
            "set_frame_current is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_key_frame_location(
        self, entity: "str|EntityInterface", frame_number: "int"
    ) -> Self:
        """
        Create an animation key-frame using the location of the entity.
        """

        print(
            "create_key_frame_location is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_key_frame_rotation(
        self, entity: "str|EntityInterface", frame_number: "int"
    ) -> Self:
        """
        Create an animation key-frame using the rotation of the entity.
        """

        print(
            "create_key_frame_rotation is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *


from codetocad.interfaces.vertex_interface import VertexInterface

from codetocad.interfaces.wire_interface import WireInterface

from codetocad.interfaces.landmark_interface import LandmarkInterface

from codetocad.interfaces.mirrorable_interface import MirrorableInterface

from codetocad.interfaces.scalable_interface import ScalableInterface

from codetocad.interfaces.landmarkable_interface import LandmarkableInterface

from codetocad.interfaces.projectable_interface import ProjectableInterface

from codetocad.interfaces.importable_interface import ImportableInterface

from codetocad.interfaces.exportable_interface import ExportableInterface

from codetocad.interfaces.patternable_interface import PatternableInterface

from codetocad.interfaces.entity_interface import EntityInterface


class SketchInterface(
    EntityInterface,
    MirrorableInterface,
    PatternableInterface,
    ImportableInterface,
    ExportableInterface,
    ScalableInterface,
    ProjectableInterface,
    LandmarkableInterface,
    metaclass=ABCMeta,
):
    """
    Capabilities related to creating and manipulating 2D sketches, composed of vertices, edges and wires.
    """

    @abstractmethod
    def __init__(
        self,
        name: "str",
        description: "str| None" = None,
        native_instance=None,
        curve_type: "CurveTypes| None" = None,
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance
        self.curve_type = curve_type

    @abstractmethod
    def get_wires(
        self,
    ) -> "list[WireInterface]":
        """
        Get a list of Wires in this Sketch.
        """

        print("get_wires is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def clone(
        self, new_name: "str", copy_landmarks: "bool" = True
    ) -> "SketchInterface":
        """
        Clone an existing sketch with its geometry and properties. Returns the new Sketch.
        """

        print("clone is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def create_text(
        self,
        text: "str",
        font_size: "str|float|Dimension" = 1.0,
        bold: "bool" = False,
        italic: "bool" = False,
        underlined: "bool" = False,
        character_spacing: "int" = 1,
        word_spacing: "int" = 1,
        line_spacing: "int" = 1,
        font_file_path: "str| None" = None,
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Adds text to a sketch.
        """

        print(
            "create_text is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_from_vertices(
        self,
        points: "list[str|list[str]|list[float]|list[Dimension]|Point|VertexInterface]",
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create a curve from 2D/3D points.
        """

        print(
            "create_from_vertices is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_point(
        self,
        point: "str|list[str]|list[float]|list[Dimension]|Point",
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create a point
        """

        print(
            "create_point is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_line(
        self,
        length: "str|float|Dimension",
        angle: "str|float|Angle",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create a line between two points
        """

        print(
            "create_line is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_line_to(
        self,
        to: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create a line between two points
        """

        print(
            "create_line_to is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_circle(
        self,
        radius: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create a circle
        """

        print(
            "create_circle is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_ellipse(
        self,
        radius_minor: "str|float|Dimension",
        radius_major: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create an ellipse
        """

        print(
            "create_ellipse is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_arc(
        self,
        end_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface",
        radius: "str|float|Dimension",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        flip: "bool| None" = False,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create an arc. The radius is the distance from the center of the circle that forms the arc, to the chord tying start_at and end_at.
        """

        print(
            "create_arc is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_rectangle(
        self,
        length: "str|float|Dimension",
        width: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create a rectangle
        """

        print(
            "create_rectangle is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_polygon(
        self,
        number_of_sides: "int",
        length: "str|float|Dimension",
        width: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create an n-gon
        """

        print(
            "create_polygon is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_trapezoid(
        self,
        length_upper: "str|float|Dimension",
        length_lower: "str|float|Dimension",
        height: "str|float|Dimension",
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create a trapezoid
        """

        print(
            "create_trapezoid is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_spiral(
        self,
        number_of_turns: "int",
        height: "str|float|Dimension",
        radius: "str|float|Dimension",
        is_clockwise: "bool" = True,
        radius_end: "str|float|Dimension| None" = None,
        center_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = None,
        options: "SketchOptions| None" = None,
    ) -> "WireInterface":
        """
        Create a spiral or helix
        """

        print(
            "create_spiral is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.entity_interface import EntityInterface


class AnalyticsInterface(metaclass=ABCMeta):
    """
    Tools for collecting data about the entities and scene.
    """

    @abstractmethod
    def measure_distance(
        self, entity1: "str|EntityInterface", entity2: "str|EntityInterface"
    ) -> "Dimensions":
        """
        The ubiquitous ruler.
        """

        print(
            "measure_distance is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def measure_angle(
        self,
        entity1: "str|EntityInterface",
        entity2: "str|EntityInterface",
        pivot: "str|EntityInterface| None" = None,
    ) -> "list[Angle]":
        """
        The ubiquitous ruler.
        """

        print(
            "measure_angle is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_world_pose(self, entity: "str|EntityInterface") -> "list[float]":
        """
        Returns the world pose of an entity.
        """

        print(
            "get_world_pose is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_bounding_box(self, entity_name: "str|EntityInterface") -> "BoundaryBox":
        """
        Returns the bounding box of an entity.
        """

        print(
            "get_bounding_box is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_dimensions(self, entity_name: "str|EntityInterface") -> "Dimensions":
        """
        Returns the dimensions of an entity.
        """

        print(
            "get_dimensions is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def log(self, message: "str") -> Self:
        """
        Write a message
        """

        print("log is called in an abstract method. Please override this method.")

        raise NotImplementedError()



from dataclasses import dataclass

from codetocad.core.angle import Angle


@dataclass
class SketchOptions:
    rotation_x: str | float | Angle | None = None
    rotation_y: str | float | Angle | None = None
    rotation_z: str | float | Angle | None = None



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.


from typing import Self


from codetocad.interfaces.animation_interface import AnimationInterface


from codetocad.interfaces.entity_interface import EntityInterface


class Animation(
    AnimationInterface,
):

    @staticmethod
    def default() -> "AnimationInterface":

        print(
            "default called",
        )

        return Animation()

    def set_frame_start(self, frame_number: "int") -> Self:

        print("set_frame_start called", f": {frame_number}")

        return self

    def set_frame_end(self, frame_number: "int") -> Self:

        print("set_frame_end called", f": {frame_number}")

        return self

    def set_frame_current(self, frame_number: "int") -> Self:

        print("set_frame_current called", f": {frame_number}")

        return self

    def create_key_frame_location(
        self, entity: "str|EntityInterface", frame_number: "int"
    ) -> Self:

        print("create_key_frame_location called", f": {entity}, {frame_number}")

        return self

    def create_key_frame_rotation(
        self, entity: "str|EntityInterface", frame_number: "int"
    ) -> Self:

        print("create_key_frame_rotation called", f": {entity}, {frame_number}")

        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.entity_interface import EntityInterface


class Entity(
    EntityInterface,
):

    def __init__(
        self, name: "str", description: "str| None" = None, native_instance=None
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance

    def is_exists(
        self,
    ) -> "bool":

        print(
            "is_exists called",
        )

        return True

    def rename(
        self, new_name: "str", renamelinked_entities_and_landmarks: "bool" = True
    ) -> Self:

        print("rename called", f": {new_name}, {renamelinked_entities_and_landmarks}")

        return self

    def delete(self, remove_children: "bool" = True) -> Self:

        print("delete called", f": {remove_children}")

        return self

    def is_visible(
        self,
    ) -> "bool":

        print(
            "is_visible called",
        )

        return True

    def set_visible(self, is_visible: "bool") -> Self:

        print("set_visible called", f": {is_visible}")

        return self

    def apply(
        self,
        rotation: "bool" = True,
        scale: "bool" = True,
        location: "bool" = False,
        modifiers: "bool" = True,
    ) -> Self:

        print("apply called", f": {rotation}, {scale}, {location}, {modifiers}")

        return self

    def get_native_instance(
        self,
    ) -> "object":

        print(
            "get_native_instance called",
        )

        return "instance"

    def get_location_world(
        self,
    ) -> "Point":

        print(
            "get_location_world called",
        )

        return Point.from_list_of_float_or_string([0, 0, 0])

    def get_location_local(
        self,
    ) -> "Point":

        print(
            "get_location_local called",
        )

        return Point.from_list_of_float_or_string([0, 0, 0])

    def select(
        self,
    ) -> Self:

        print(
            "select called",
        )

        return self

    def translate_xyz(
        self,
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> Self:

        print("translate_xyz called", f": {x}, {y}, {z}")

        return self

    def translate_x(self, amount: "str|float|Dimension") -> Self:

        print("translate_x called", f": {amount}")

        return self

    def translate_y(self, amount: "str|float|Dimension") -> Self:

        print("translate_y called", f": {amount}")

        return self

    def translate_z(self, amount: "str|float|Dimension") -> Self:

        print("translate_z called", f": {amount}")

        return self

    def rotate_xyz(
        self, x: "str|float|Angle", y: "str|float|Angle", z: "str|float|Angle"
    ) -> Self:

        print("rotate_xyz called", f": {x}, {y}, {z}")

        return self

    def rotate_x(self, rotation: "str|float|Angle") -> Self:

        print("rotate_x called", f": {rotation}")

        return self

    def rotate_y(self, rotation: "str|float|Angle") -> Self:

        print("rotate_y called", f": {rotation}")

        return self

    def rotate_z(self, rotation: "str|float|Angle") -> Self:

        print("rotate_z called", f": {rotation}")

        return self

    def get_bounding_box(
        self,
    ) -> "BoundaryBox":

        print(
            "get_bounding_box called",
        )

        return BoundaryBox(BoundaryAxis(0, 0), BoundaryAxis(0, 0), BoundaryAxis(0, 0))

    def get_dimensions(
        self,
    ) -> "Dimensions":

        print(
            "get_dimensions called",
        )

        return Dimensions.from_point(Point.from_list_of_float_or_string([0, 0, 0]))



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.


from typing import Self


from codetocad.interfaces.render_interface import RenderInterface


from codetocad.interfaces.camera_interface import CameraInterface


class Render(
    RenderInterface,
):

    def render_image(
        self,
        output_file_path: "str",
        overwrite: "bool" = True,
        file_type: "str| None" = None,
    ) -> Self:

        print("render_image called", f": {output_file_path}, {overwrite}, {file_type}")

        return self

    def render_video_mp4(
        self,
        output_file_path: "str",
        start_frame_number: "int" = 1,
        end_frame_number: "int" = 100,
        step_frames: "int" = 1,
        overwrite: "bool" = True,
    ) -> Self:

        print(
            "render_video_mp4 called",
            f": {output_file_path}, {start_frame_number}, {end_frame_number}, {step_frames}, {overwrite}",
        )

        return self

    def render_video_frames(
        self,
        output_folder_path: "str",
        file_name_prefix: "str",
        start_frame_number: "int" = 1,
        end_frame_number: "int" = 100,
        step_frames: "int" = 1,
        overwrite: "bool" = True,
        file_type: "str| None" = None,
    ) -> Self:

        print(
            "render_video_frames called",
            f": {output_folder_path}, {file_name_prefix}, {start_frame_number}, {end_frame_number}, {step_frames}, {overwrite}, {file_type}",
        )

        return self

    def set_frame_rate(self, frame_rate: "int") -> Self:

        print("set_frame_rate called", f": {frame_rate}")

        return self

    def set_resolution(self, x: "int", y: "int") -> Self:

        print("set_resolution called", f": {x}, {y}")

        return self

    def set_render_quality(self, quality: "int") -> Self:

        print("set_render_quality called", f": {quality}")

        return self

    def set_render_engine(self, name: "str") -> Self:

        print("set_render_engine called", f": {name}")

        return self

    def set_camera(self, camera_name_or_instance: "str|CameraInterface") -> Self:

        print("set_camera called", f": {camera_name_or_instance}")

        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.entity_interface import EntityInterface


class PatternableInterface(metaclass=ABCMeta):
    """
    An entity that can be patterned.
    """

    @abstractmethod
    def linear_pattern(
        self,
        instance_count: "int",
        offset: "str|float|Dimension",
        direction_axis: "str|int|Axis" = "z",
    ) -> Self:
        """
        Pattern in a uniform direction.
        """

        print(
            "linear_pattern is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: "str|float|Angle",
        center_entity_or_landmark: "str|EntityInterface",
        normal_direction_axis: "str|int|Axis" = "z",
    ) -> Self:
        """
        Pattern in a circular direction.
        """

        print(
            "circular_pattern is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.


from typing import Self


from codetocad.interfaces.camera_interface import CameraInterface


from providers.sample.entity import Entity


class Camera(CameraInterface, Entity):

    def __init__(
        self, name: "str", description: "str| None" = None, native_instance=None
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance

    def create_perspective(
        self,
    ) -> Self:

        print(
            "create_perspective called",
        )

        return self

    def create_orthogonal(
        self,
    ) -> Self:

        print(
            "create_orthogonal called",
        )

        return self

    def create_panoramic(
        self,
    ) -> Self:

        print(
            "create_panoramic called",
        )

        return self

    def set_focal_length(self, length: "float") -> Self:

        print("set_focal_length called", f": {length}")

        return self



# THIS IS AN AUTO-GENERATED FILE. DO NOT CHANGE.

from .entity import Entity
from codetocad.interfaces.entity_interface import EntityInterface
from .part import Part
from codetocad.interfaces.part_interface import PartInterface
from .sketch import Sketch
from codetocad.interfaces.sketch_interface import SketchInterface
from .vertex import Vertex
from codetocad.interfaces.vertex_interface import VertexInterface
from .edge import Edge
from codetocad.interfaces.edge_interface import EdgeInterface
from .wire import Wire
from codetocad.interfaces.wire_interface import WireInterface
from .landmark import Landmark
from codetocad.interfaces.landmark_interface import LandmarkInterface
from .joint import Joint
from codetocad.interfaces.joint_interface import JointInterface
from .material import Material
from codetocad.interfaces.material_interface import MaterialInterface
from .animation import Animation
from codetocad.interfaces.animation_interface import AnimationInterface
from .light import Light
from codetocad.interfaces.light_interface import LightInterface
from .camera import Camera
from codetocad.interfaces.camera_interface import CameraInterface
from .render import Render
from codetocad.interfaces.render_interface import RenderInterface
from .scene import Scene
from codetocad.interfaces.scene_interface import SceneInterface
from .analytics import Analytics
from codetocad.interfaces.analytics_interface import AnalyticsInterface


def register():
    __import__("codetocad").providers.register(Entity, EntityInterface)
    __import__("codetocad").providers.register(Part, PartInterface)
    __import__("codetocad").providers.register(Sketch, SketchInterface)
    __import__("codetocad").providers.register(Vertex, VertexInterface)
    __import__("codetocad").providers.register(Edge, EdgeInterface)
    __import__("codetocad").providers.register(Wire, WireInterface)
    __import__("codetocad").providers.register(Landmark, LandmarkInterface)
    __import__("codetocad").providers.register(Joint, JointInterface)
    __import__("codetocad").providers.register(Material, MaterialInterface)
    __import__("codetocad").providers.register(Animation, AnimationInterface)
    __import__("codetocad").providers.register(Light, LightInterface)
    __import__("codetocad").providers.register(Camera, CameraInterface)
    __import__("codetocad").providers.register(Render, RenderInterface)
    __import__("codetocad").providers.register(Scene, SceneInterface)
    __import__("codetocad").providers.register(Analytics, AnalyticsInterface)



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from typing import Self


from codetocad.interfaces.entity_interface import EntityInterface


class LightInterface(EntityInterface, metaclass=ABCMeta):
    """
    Manipulate a light object.
    """

    @abstractmethod
    def __init__(
        self, name: "str", description: "str| None" = None, native_instance=None
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def set_color(
        self, r_value: "int|float", g_value: "int|float", b_value: "int|float"
    ) -> Self:
        """
        Set the color of an existing light.
        """

        print("set_color is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def create_sun(self, energy_level: "float") -> Self:
        """
        Create a Sun-type light.
        """

        print(
            "create_sun is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_spot(self, energy_level: "float") -> Self:
        """
        Create a Spot-type light.
        """

        print(
            "create_spot is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_point(self, energy_level: "float") -> Self:
        """
        Create a Point-type light.
        """

        print(
            "create_point is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_area(self, energy_level: "float") -> Self:
        """
        Create an Area-type light.
        """

        print(
            "create_area is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



import inspect
import unittest
import json

capabilities_json = "codetocad/capabilities.json"

capabilies_class_names = []

with open(capabilities_json) as f:
    capabilities: dict = json.load(f)
    for capability_name, value in capabilities["capabilities"].items():
        if not value.get("is_interface_only", False):
            capabilies_class_names.append(capability_name)


class TestProviderCompliance(unittest.TestCase):
    def test_no_abstract_provider(self):
        print("capabilies_class_names", capabilies_class_names)
        all_providers_import = __import__("providers")
        all_providers_modules = inspect.getmembers(
            all_providers_import, predicate=inspect.ismodule
        )
        for module_name, provider_module in all_providers_modules:
            print(f"Testing module {module_name}")
            all_classes = inspect.getmembers(provider_module, predicate=inspect.isclass)
            for class_name, some_class in all_classes:
                if class_name not in capabilies_class_names:
                    continue
                abstract_methods = list(some_class.__abstractmethods__)
                assert (
                    len(abstract_methods) == 0
                ), f"{class_name} has abstract methods {abstract_methods} in the {module_name} provider."



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.part_interface import PartInterface


from codetocad.interfaces.landmark_interface import LandmarkInterface

from codetocad.interfaces.material_interface import MaterialInterface


from codetocad.interfaces.booleanable_interface import BooleanableInterface


from codetocad.interfaces.entity_interface import EntityInterface


from codetocad.proxy.landmark import Landmark


from providers.sample.entity import Entity


class Part(PartInterface, Entity):

    def __init__(
        self, name: "str", description: "str| None" = None, native_instance=None
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance

    def create_cube(
        self,
        width: "str|float|Dimension",
        length: "str|float|Dimension",
        height: "str|float|Dimension",
        options: "PartOptions| None" = None,
    ) -> Self:

        print("create_cube called", f": {width}, {length}, {height}, {options}")

        return self

    def create_cone(
        self,
        radius: "str|float|Dimension",
        height: "str|float|Dimension",
        draft_radius: "str|float|Dimension" = 0,
        options: "PartOptions| None" = None,
    ) -> Self:

        print("create_cone called", f": {radius}, {height}, {draft_radius}, {options}")

        return self

    def create_cylinder(
        self,
        radius: "str|float|Dimension",
        height: "str|float|Dimension",
        options: "PartOptions| None" = None,
    ) -> Self:

        print("create_cylinder called", f": {radius}, {height}, {options}")

        return self

    def create_torus(
        self,
        inner_radius: "str|float|Dimension",
        outer_radius: "str|float|Dimension",
        options: "PartOptions| None" = None,
    ) -> Self:

        print("create_torus called", f": {inner_radius}, {outer_radius}, {options}")

        return self

    def create_sphere(
        self, radius: "str|float|Dimension", options: "PartOptions| None" = None
    ) -> Self:

        print("create_sphere called", f": {radius}, {options}")

        return self

    def create_gear(
        self,
        outer_radius: "str|float|Dimension",
        addendum: "str|float|Dimension",
        inner_radius: "str|float|Dimension",
        dedendum: "str|float|Dimension",
        height: "str|float|Dimension",
        pressure_angle: "str|float|Angle" = "20d",
        number_of_teeth: "int" = 12,
        skew_angle: "str|float|Angle" = 0,
        conical_angle: "str|float|Angle" = 0,
        crown_angle: "str|float|Angle" = 0,
        options: "PartOptions| None" = None,
    ) -> Self:

        print(
            "create_gear called",
            f": {outer_radius}, {addendum}, {inner_radius}, {dedendum}, {height}, {pressure_angle}, {number_of_teeth}, {skew_angle}, {conical_angle}, {crown_angle}, {options}",
        )

        return self

    def clone(self, new_name: "str", copy_landmarks: "bool" = True) -> "PartInterface":

        print("clone called", f": {new_name}, {copy_landmarks}")

        return Part("a part")

    def hollow(
        self,
        thickness_x: "str|float|Dimension",
        thickness_y: "str|float|Dimension",
        thickness_z: "str|float|Dimension",
        start_axis: "str|int|Axis" = "z",
        flip_axis: "bool" = False,
    ) -> Self:

        print(
            "hollow called",
            f": {thickness_x}, {thickness_y}, {thickness_z}, {start_axis}, {flip_axis}",
        )

        return self

    def thicken(self, radius: "str|float|Dimension") -> Self:

        print("thicken called", f": {radius}")

        return self

    def hole(
        self,
        hole_landmark: "str|LandmarkInterface",
        radius: "str|float|Dimension",
        depth: "str|float|Dimension",
        normal_axis: "str|int|Axis" = "z",
        flip_axis: "bool" = False,
        initial_rotation_x: "str|float|Angle" = 0.0,
        initial_rotation_y: "str|float|Angle" = 0.0,
        initial_rotation_z: "str|float|Angle" = 0.0,
        mirror_about_entity_or_landmark: "str|EntityInterface| None" = None,
        mirror_axis: "str|int|Axis" = "x",
        mirror: "bool" = False,
        circular_pattern_instance_count: "int" = 1,
        circular_pattern_instance_separation: "str|float|Angle" = 0.0,
        circular_pattern_instance_axis: "str|int|Axis" = "z",
        circular_pattern_about_entity_or_landmark: "str|EntityInterface| None" = None,
        linear_pattern_instance_count: "int" = 1,
        linear_pattern_instance_separation: "str|float|Dimension" = 0.0,
        linear_pattern_instance_axis: "str|int|Axis" = "x",
        linear_pattern2nd_instance_count: "int" = 1,
        linear_pattern2nd_instance_separation: "str|float|Dimension" = 0.0,
        linear_pattern2nd_instance_axis: "str|int|Axis" = "y",
    ) -> Self:

        print(
            "hole called",
            f": {hole_landmark}, {radius}, {depth}, {normal_axis}, {flip_axis}, {initial_rotation_x}, {initial_rotation_y}, {initial_rotation_z}, {mirror_about_entity_or_landmark}, {mirror_axis}, {mirror}, {circular_pattern_instance_count}, {circular_pattern_instance_separation}, {circular_pattern_instance_axis}, {circular_pattern_about_entity_or_landmark}, {linear_pattern_instance_count}, {linear_pattern_instance_separation}, {linear_pattern_instance_axis}, {linear_pattern2nd_instance_count}, {linear_pattern2nd_instance_separation}, {linear_pattern2nd_instance_axis}",
        )

        return self

    def twist(
        self,
        angle: "str|float|Angle",
        screw_pitch: "str|float|Dimension",
        iterations: "int" = 1,
        axis: "str|int|Axis" = "z",
    ) -> Self:

        print("twist called", f": {angle}, {screw_pitch}, {iterations}, {axis}")

        return self

    def set_material(self, material_name: "str|MaterialInterface") -> Self:

        print("set_material called", f": {material_name}")

        return self

    def is_colliding_with_part(self, other_part: "str|PartInterface") -> "bool":

        print("is_colliding_with_part called", f": {other_part}")

        return True

    def fillet_all_edges(
        self, radius: "str|float|Dimension", use_width: "bool" = False
    ) -> Self:

        print("fillet_all_edges called", f": {radius}, {use_width}")

        return self

    def fillet_edges(
        self,
        radius: "str|float|Dimension",
        landmarks_near_edges: "list[str|LandmarkInterface]",
        use_width: "bool" = False,
    ) -> Self:

        print("fillet_edges called", f": {radius}, {landmarks_near_edges}, {use_width}")

        return self

    def fillet_faces(
        self,
        radius: "str|float|Dimension",
        landmarks_near_faces: "list[str|LandmarkInterface]",
        use_width: "bool" = False,
    ) -> Self:

        print("fillet_faces called", f": {radius}, {landmarks_near_faces}, {use_width}")

        return self

    def chamfer_all_edges(self, radius: "str|float|Dimension") -> Self:

        print("chamfer_all_edges called", f": {radius}")

        return self

    def chamfer_edges(
        self,
        radius: "str|float|Dimension",
        landmarks_near_edges: "list[str|LandmarkInterface]",
    ) -> Self:

        print("chamfer_edges called", f": {radius}, {landmarks_near_edges}")

        return self

    def chamfer_faces(
        self,
        radius: "str|float|Dimension",
        landmarks_near_faces: "list[str|LandmarkInterface]",
    ) -> Self:

        print("chamfer_faces called", f": {radius}, {landmarks_near_faces}")

        return self

    def select_vertex_near_landmark(
        self, landmark_name: "str|LandmarkInterface| None" = None
    ) -> Self:

        print("select_vertex_near_landmark called", f": {landmark_name}")

        return self

    def select_edge_near_landmark(
        self, landmark_name: "str|LandmarkInterface| None" = None
    ) -> Self:

        print("select_edge_near_landmark called", f": {landmark_name}")

        return self

    def select_face_near_landmark(
        self, landmark_name: "str|LandmarkInterface| None" = None
    ) -> Self:

        print("select_face_near_landmark called", f": {landmark_name}")

        return self

    def mirror(
        self,
        mirror_across_entity: "str|EntityInterface",
        axis: "str|int|Axis",
        resulting_mirrored_entity_name: "str| None" = None,
    ) -> Self:

        print(
            "mirror called",
            f": {mirror_across_entity}, {axis}, {resulting_mirrored_entity_name}",
        )

        return self

    def linear_pattern(
        self,
        instance_count: "int",
        offset: "str|float|Dimension",
        direction_axis: "str|int|Axis" = "z",
    ) -> Self:

        print(
            "linear_pattern called", f": {instance_count}, {offset}, {direction_axis}"
        )

        return self

    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: "str|float|Angle",
        center_entity_or_landmark: "str|EntityInterface",
        normal_direction_axis: "str|int|Axis" = "z",
    ) -> Self:

        print(
            "circular_pattern called",
            f": {instance_count}, {separation_angle}, {center_entity_or_landmark}, {normal_direction_axis}",
        )

        return self

    def remesh(self, strategy: "str", amount: "float") -> Self:

        print("remesh called", f": {strategy}, {amount}")

        return self

    def subdivide(self, amount: "float") -> Self:

        print("subdivide called", f": {amount}")

        return self

    def decimate(self, amount: "float") -> Self:

        print("decimate called", f": {amount}")

        return self

    def create_from_file(self, file_path: "str", file_type: "str| None" = None) -> Self:

        print("create_from_file called", f": {file_path}, {file_type}")

        return self

    def export(
        self, file_path: "str", overwrite: "bool" = True, scale: "float" = 1.0
    ) -> Self:

        print("export called", f": {file_path}, {overwrite}, {scale}")

        return self

    def scale_xyz(
        self,
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> Self:

        print("scale_xyz called", f": {x}, {y}, {z}")

        return self

    def scale_x(self, scale: "str|float|Dimension") -> Self:

        print("scale_x called", f": {scale}")

        return self

    def scale_y(self, scale: "str|float|Dimension") -> Self:

        print("scale_y called", f": {scale}")

        return self

    def scale_z(self, scale: "str|float|Dimension") -> Self:

        print("scale_z called", f": {scale}")

        return self

    def scale_x_by_factor(self, scale_factor: "float") -> Self:

        print("scale_x_by_factor called", f": {scale_factor}")

        return self

    def scale_y_by_factor(self, scale_factor: "float") -> Self:

        print("scale_y_by_factor called", f": {scale_factor}")

        return self

    def scale_z_by_factor(self, scale_factor: "float") -> Self:

        print("scale_z_by_factor called", f": {scale_factor}")

        return self

    def scale_keep_aspect_ratio(
        self, scale: "str|float|Dimension", axis: "str|int|Axis"
    ) -> Self:

        print("scale_keep_aspect_ratio called", f": {scale}, {axis}")

        return self

    def create_landmark(
        self,
        landmark_name: "str",
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> "LandmarkInterface":

        print("create_landmark called", f": {landmark_name}, {x}, {y}, {z}")

        return Landmark("name", "parent")

    def get_landmark(self, landmark_name: "str|PresetLandmark") -> "LandmarkInterface":

        print("get_landmark called", f": {landmark_name}")

        return Landmark("name", "parent")

    def union(
        self,
        other: "str|BooleanableInterface",
        delete_after_union: "bool" = True,
        is_transfer_data: "bool" = False,
    ) -> Self:

        print("union called", f": {other}, {delete_after_union}, {is_transfer_data}")

        return self

    def subtract(
        self,
        other: "str|BooleanableInterface",
        delete_after_subtract: "bool" = True,
        is_transfer_data: "bool" = False,
    ) -> Self:

        print(
            "subtract called", f": {other}, {delete_after_subtract}, {is_transfer_data}"
        )

        return self

    def intersect(
        self,
        other: "str|BooleanableInterface",
        delete_after_intersect: "bool" = True,
        is_transfer_data: "bool" = False,
    ) -> Self:

        print(
            "intersect called",
            f": {other}, {delete_after_intersect}, {is_transfer_data}",
        )

        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from typing import Self


class BooleanableInterface(metaclass=ABCMeta):
    """
    An entity that supports boolean operations: union, intersect, subtract.
    """

    @abstractmethod
    def union(
        self,
        other: "str|BooleanableInterface",
        delete_after_union: "bool" = True,
        is_transfer_data: "bool" = False,
    ) -> Self:
        """
        Boolean union
        """

        print("union is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def subtract(
        self,
        other: "str|BooleanableInterface",
        delete_after_subtract: "bool" = True,
        is_transfer_data: "bool" = False,
    ) -> Self:
        """
        Boolean subtraction
        """

        print("subtract is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def intersect(
        self,
        other: "str|BooleanableInterface",
        delete_after_intersect: "bool" = True,
        is_transfer_data: "bool" = False,
    ) -> Self:
        """
        Boolean intersection
        """

        print("intersect is called in an abstract method. Please override this method.")

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.vertex_interface import VertexInterface

from codetocad.interfaces.part_interface import PartInterface

from codetocad.interfaces.landmark_interface import LandmarkInterface

from codetocad.interfaces.edge_interface import EdgeInterface

from codetocad.interfaces.landmarkable_interface import LandmarkableInterface

from codetocad.interfaces.subdividable_interface import SubdividableInterface

from codetocad.interfaces.booleanable_interface import BooleanableInterface

from codetocad.interfaces.projectable_interface import ProjectableInterface

from codetocad.interfaces.mirrorable_interface import MirrorableInterface

from codetocad.interfaces.patternable_interface import PatternableInterface

from codetocad.interfaces.entity_interface import EntityInterface


class WireInterface(
    EntityInterface,
    MirrorableInterface,
    PatternableInterface,
    ProjectableInterface,
    LandmarkableInterface,
    BooleanableInterface,
    SubdividableInterface,
    metaclass=ABCMeta,
):
    """
    A collection of connected edges.
    """

    @abstractmethod
    def __init__(
        self,
        name: "str",
        edges: "list[EdgeInterface]",
        description: "str| None" = None,
        native_instance=None,
        parent_entity: "str|EntityInterface| None" = None,
    ):

        self.name = name
        self.edges = edges
        self.description = description
        self.native_instance = native_instance
        self.parent_entity = parent_entity

    @abstractmethod
    def get_normal(self, flip: "bool| None" = False) -> "Point":
        """
        Get the normal created by this wire. Must be a closed wire.
        """

        print(
            "get_normal is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_edges(
        self,
    ) -> "list[EdgeInterface]":
        """
        Return references to all the edges making up this wire.
        """

        print("get_edges is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def get_vertices(
        self,
    ) -> "list[VertexInterface]":
        """
        Collapse all edges' vertices into one list.
        """

        print(
            "get_vertices is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_is_closed(
        self,
    ) -> "bool":
        """
        Checks if a wire is closed. Note: A closed wire is a Face or Surface.
        """

        print(
            "get_is_closed is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def loft(
        self, other: "WireInterface", new_part_name: "str| None" = None
    ) -> "PartInterface":
        """
        Create a surface between two Wires (Faces). If new_part_name is not provided, the two Wires' parents and the surface will be boolean union'ed, and the resulting Part will take the name of the first wire.
        """

        print("loft is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def revolve(
        self,
        angle: "str|float|Angle",
        about_entity_or_landmark: "str|EntityInterface",
        axis: "str|int|Axis" = "z",
    ) -> "PartInterface":
        """
        Revolve a Wire around another Entity or Landmark
        """

        print("revolve is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def twist(
        self,
        angle: "str|float|Angle",
        screw_pitch: "str|float|Dimension",
        iterations: "int" = 1,
        axis: "str|int|Axis" = "z",
    ) -> Self:
        """
        AKA Helix, Screw.
        """

        print("twist is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def extrude(self, length: "str|float|Dimension") -> "PartInterface":
        """
        Extrude a curve by a specified length. Returns a Part type.
        """

        print("extrude is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def sweep(
        self, profile_name_or_instance: "str|WireInterface", fill_cap: "bool" = True
    ) -> "PartInterface":
        """
        Extrude this Wire along the path of another Wire
        """

        print("sweep is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def offset(self, radius: "str|float|Dimension") -> "WireInterface":
        """
        Uniformly add a wall around a Wire. This returns a new wire in the same sketch.
        """

        print("offset is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def profile(self, profile_curve_name: "str") -> Self:
        """
        Bend this curve along the path of another
        """

        print("profile is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def create_from_vertices(
        self,
        points: "list[str|list[str]|list[float]|list[Dimension]|Point|VertexInterface]",
        options: "SketchOptions| None" = None,
    ) -> Self:
        """
        Create a curve from 2D/3D points.
        """

        print(
            "create_from_vertices is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_point(
        self,
        point: "str|list[str]|list[float]|list[Dimension]|Point",
        options: "SketchOptions| None" = None,
    ) -> Self:
        """
        Create a point
        """

        print(
            "create_point is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_line(
        self,
        length: "str|float|Dimension",
        angle: "str|float|Angle",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        options: "SketchOptions| None" = None,
    ) -> Self:
        """
        Create a line between two points
        """

        print(
            "create_line is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_line_to(
        self,
        to: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        options: "SketchOptions| None" = None,
    ) -> Self:
        """
        Create a line between two points
        """

        print(
            "create_line_to is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_arc(
        self,
        end_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface",
        radius: "str|float|Dimension",
        start_at: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface|LandmarkInterface|PresetLandmark| None" = "PresetLandmark.end",
        flip: "bool| None" = False,
        options: "SketchOptions| None" = None,
    ) -> Self:
        """
        Create an arc. The radius is the distance from the center of the circle that forms the arc, to the chord tying start_at and end_at.
        """

        print(
            "create_arc is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from typing import Self


class SubdividableInterface(metaclass=ABCMeta):
    """
    An entity that can be broken down or scaled up into more components.
    """

    @abstractmethod
    def remesh(self, strategy: "str", amount: "float") -> Self:
        """
        Remeshing changes the shape of an entity.
        """

        print("remesh is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def subdivide(self, amount: "float") -> Self:
        """
        Subdivide an entity into more components.
        """

        print("subdivide is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def decimate(self, amount: "float") -> Self:
        """
        Decimate an entity into less components.
        """

        print("decimate is called in an abstract method. Please override this method.")

        raise NotImplementedError()



from dataclasses import dataclass
from functools import reduce
from typing import TYPE_CHECKING, List, Tuple
from codetocad.core.dimension import Dimension
from codetocad.enums.length_unit import LengthUnit


if TYPE_CHECKING:
    from codetocad.interfaces.vertex_interface import VertexInterface


@dataclass
class Point:
    x: Dimension
    y: Dimension
    z: Dimension

    def to_list(self) -> list[Dimension]:
        return [self.x, self.y, self.z]

    def to_tuple(self) -> Tuple[Dimension, Dimension, Dimension]:
        return (self.x, self.y, self.z)

    def to_tuple_float(self, convert_to_unit: LengthUnit) -> Tuple[float, float, float]:
        return (
            self.x.convert_to_unit(convert_to_unit).value,
            self.y.convert_to_unit(convert_to_unit).value,
            self.z.convert_to_unit(convert_to_unit).value,
        )

    def magnitude(self) -> Dimension:
        return reduce(
            lambda acc, value: value.raise_power(2) + acc,
            self.to_list(),
            Dimension.zero(),
        ).raise_power(1 / 2)

    def distance_to(self, other: "Point") -> Dimension:
        return (other - self).magnitude()

    def is_touching(
        self,
        other: "Point",
        tolerance: float = 0.001,
    ) -> bool:
        return self.distance_to(other).value < tolerance

    @staticmethod
    def from_list(point_list: List[Dimension]) -> "Point":
        assert len(point_list) == 3, "Point list must contain three Dimensions."
        return Point(point_list[0], point_list[1], point_list[2])

    @staticmethod
    def from_list_of_float_or_string(
        point_representation: "str|list[str]|list[float]|list[Dimension]|Point",
    ) -> "Point":
        if isinstance(point_representation, (list, tuple)):
            assert (
                len(point_representation) == 3
            ), "Point list must contain three Dimensions."
            points = [
                Dimension.from_dimension_or_its_float_or_string_value(point, None)
                for point in point_representation
            ]
            return Point(points[0], points[1], points[2])
        elif isinstance(point_representation, str):
            from codetocad.utilities import get_dimension_list_from_string_list

            points = get_dimension_list_from_string_list(point_representation)
            return Point(points[0], points[1], points[2])
        elif isinstance(point_representation, Point):
            return point_representation

        raise ValueError(f"Cannot convert type {type(point_representation)} to Point.")

    @staticmethod
    def from_list_of_float_or_string_or_Vertex(
        point_representation: "str|list[str]|list[float]|list[Dimension]|Point|VertexInterface",
    ) -> "Point":
        from codetocad.interfaces.vertex_interface import VertexInterface

        if isinstance(point_representation, VertexInterface):
            return point_representation.location
        else:
            return Point.from_list_of_float_or_string(point_representation)

    def arithmetic_precheck_and_unit_conversion(self, other) -> "Point":
        assert other is not None, "Right-hand value cannot be None."

        if not isinstance(other, (int, float, str, Dimension, list, Point)):
            raise TypeError(
                "Only int/float, Dimension, or Dimension String, or a list of those types is allowed."
            )

        if isinstance(other, (int, float)):
            return Point(Dimension(other), Dimension(other), Dimension(other))

        x = Dimension(0)
        y = Dimension(0)
        z = Dimension(0)

        from codetocad.utilities import get_dimension_list_from_string_list

        if isinstance(other, list):
            [x, y, z] = get_dimension_list_from_string_list(other)

        if isinstance(other, str):
            if "," in other:
                [x, y, z] = get_dimension_list_from_string_list(other)
            else:
                other = Dimension.from_string(other)

        if isinstance(other, Dimension):
            x = other
            y = other
            z = other

        if isinstance(other, Point):
            x = other.x
            y = other.y
            z = other.z

        if x.unit is not None and self.x.unit is not None and x.unit != self.x.unit:
            x: Dimension = x.convert_to_unit(self.x.unit)
        if y.unit is not None and self.y.unit is not None and y.unit != self.y.unit:
            y: Dimension = y.convert_to_unit(self.y.unit)
        if z.unit is not None and self.z.unit is not None and z.unit != self.z.unit:
            z: Dimension = z.convert_to_unit(self.z.unit)
        return Point(x, y, z)

    def __eq__(self, other) -> bool:
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return self.x == other.x and self.y == other.y and self.z == other.z

    def __lt__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return self.x < other.x and self.y < other.y and self.z < other.z

    def __le__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return self.x <= other.x and self.y <= other.y and self.z <= other.z

    def __gt__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return self.x > other.x and self.y > other.y and self.z > other.z

    def __ge__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return self.x >= other.x and self.y >= other.y and self.z >= other.z

    def __add__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        x = self.x + other.x
        y = self.y + other.y
        z = self.z + other.z
        return Point(x, y, z)

    def __sub__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        x = self.x - other.x
        y = self.y - other.y
        z = self.z - other.z
        return Point(x, y, z)

    def __mul__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        x = self.x * other.x
        y = self.y * other.y
        z = self.z * other.z
        return Point(x, y, z)

    def __truediv__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        x = self.x / other.x
        y = self.y / other.y
        z = self.z / other.z
        return Point(x, y, z)

    def __floordiv__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        x = self.x // other.x
        y = self.y // other.y
        z = self.z // other.z
        return Point(x, y, z)

    def __mod__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        x = self.x % other.x
        y = self.y % other.y
        z = self.z % other.z
        return Point(x, y, z)

    # def __divmod__(self, other):
    #     other = self.arithmetic_precheck_and_unit_conversion(other)
    #     x = divmod(self.x, other.x)
    #     y = divmod(self.y, other.y)
    #     z = divmod(self.z, other.z)
    #     return Point(x, y, z)

    def __pow__(self, other, mod=None):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        x = pow(self.x, other.x)
        y = pow(self.y, other.y)
        z = pow(self.z, other.z)
        return Point(x, y, z)

    def __abs__(self):
        x = abs(self.x)
        y = abs(self.y)
        z = abs(self.z)
        return Point(x, y, z)

    def __copy__(self):
        return self.__deepcopy__()

    def __deepcopy__(self):
        return Point(self.x, self.y, self.z)

    def copy(self):
        return self.__deepcopy__()

    def __getitem__(self, key):
        if key == 0:
            return self.x
        if key == 1:
            return self.y
        if key == 2:
            return self.z

    def __str__(self):
        return f"""x   y   z
{self.x}  {self.y}  {self.z}
"""

    def __repr__(self) -> str:
        return self.__str__()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.entity_interface import EntityInterface


class JointInterface(metaclass=ABCMeta):
    """
    Joints define the relationships and constraints between entities.
    """

    @abstractmethod
    def __init__(self, entity1: "str|EntityInterface", entity2: "str|EntityInterface"):

        self.entity1 = entity1
        self.entity2 = entity2

    @abstractmethod
    def translate_landmark_onto_another(
        self,
    ) -> Self:
        """
        Transforms one landmark onto another
        """

        print(
            "translate_landmark_onto_another is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def pivot(
        self,
    ) -> Self:
        """
        Constraint the rotation origin of entity B to entity A's landmark.
        """

        print("pivot is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def gear_ratio(self, ratio: "float") -> Self:
        """
        Constraint the rotation of entity B to be a percentage of entity A's
        """

        print(
            "gear_ratio is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def limit_location_xyz(
        self,
        x: "str|float|Dimension| None" = None,
        y: "str|float|Dimension| None" = None,
        z: "str|float|Dimension| None" = None,
    ) -> Self:
        """
        Constraint the translation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_location_xyz is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def limit_location_x(
        self,
        min: "str|float|Dimension| None" = None,
        max: "str|float|Dimension| None" = None,
    ) -> Self:
        """
        Constraint the translation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_location_x is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def limit_location_y(
        self,
        min: "str|float|Dimension| None" = None,
        max: "str|float|Dimension| None" = None,
    ) -> Self:
        """
        Constraint the translation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_location_y is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def limit_location_z(
        self,
        min: "str|float|Dimension| None" = None,
        max: "str|float|Dimension| None" = None,
    ) -> Self:
        """
        Constraint the translation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_location_z is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def limit_rotation_xyz(
        self,
        x: "str|float|Angle| None" = None,
        y: "str|float|Angle| None" = None,
        z: "str|float|Angle| None" = None,
    ) -> Self:
        """
        Constraint the rotation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_rotation_xyz is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def limit_rotation_x(
        self, min: "str|float|Angle| None" = None, max: "str|float|Angle| None" = None
    ) -> Self:
        """
        Constraint the rotation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_rotation_x is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def limit_rotation_y(
        self, min: "str|float|Angle| None" = None, max: "str|float|Angle| None" = None
    ) -> Self:
        """
        Constraint the rotation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_rotation_y is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def limit_rotation_z(
        self, min: "str|float|Angle| None" = None, max: "str|float|Angle| None" = None
    ) -> Self:
        """
        Constraint the rotation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_rotation_z is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



__providers = {}


def register(provider, interface):
    global __providers
    __providers[interface] = provider


def get_provider(interface) -> type:
    global __providers
    return __providers[interface]



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.analytics_interface import AnalyticsInterface


from codetocad.interfaces.entity_interface import EntityInterface


class Analytics(
    AnalyticsInterface,
):

    def measure_distance(
        self, entity1: "str|EntityInterface", entity2: "str|EntityInterface"
    ) -> "Dimensions":

        print("measure_distance called", f": {entity1}, {entity2}")

        return Dimensions.from_point(Point.from_list_of_float_or_string([0, 0, 0]))

    def measure_angle(
        self,
        entity1: "str|EntityInterface",
        entity2: "str|EntityInterface",
        pivot: "str|EntityInterface| None" = None,
    ) -> "list[Angle]":

        print("measure_angle called", f": {entity1}, {entity2}, {pivot}")

        return [Angle(90)]

    def get_world_pose(self, entity: "str|EntityInterface") -> "list[float]":

        print("get_world_pose called", f": {entity}")

        return [0.0]

    def get_bounding_box(self, entity_name: "str|EntityInterface") -> "BoundaryBox":

        print("get_bounding_box called", f": {entity_name}")

        return BoundaryBox(BoundaryAxis(0, 0), BoundaryAxis(0, 0), BoundaryAxis(0, 0))

    def get_dimensions(self, entity_name: "str|EntityInterface") -> "Dimensions":

        print("get_dimensions called", f": {entity_name}")

        return Dimensions.from_point(Point.from_list_of_float_or_string([0, 0, 0]))

    def log(self, message: "str") -> Self:

        print("log called", f": {message}")

        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.material_interface import MaterialInterface


class Material(
    MaterialInterface,
):

    def __init__(self, name: "str", description: "str| None" = None):

        self.name = name
        self.description = description

    @staticmethod
    def get_preset(material_name: "PresetMaterial") -> "MaterialInterface":

        print("get_preset called", f": {material_name}")

        return Material("mat")

    def set_color(
        self,
        r_value: "int|float",
        g_value: "int|float",
        b_value: "int|float",
        a_value: "int|float" = 1.0,
    ) -> Self:

        print("set_color called", f": {r_value}, {g_value}, {b_value}, {a_value}")

        return self

    def set_reflectivity(self, reflectivity: "float") -> Self:

        print("set_reflectivity called", f": {reflectivity}")

        return self

    def set_roughness(self, roughness: "float") -> Self:

        print("set_roughness called", f": {roughness}")

        return self

    def set_image_texture(self, image_file_path: "str") -> Self:

        print("set_image_texture called", f": {image_file_path}")

        return self



import os
import subprocess
from codetocad.launchers.launcher_args import LauncherArgs


def run_unknown_process(launcher_args: LauncherArgs):
    """
    Launches a process as described by LauncherArgs::to_subprocess_args().
    """
    return subprocess.Popen(launcher_args.to_subprocess_args(), env=os.environ)



import re
from typing import Optional, Union
from codetocad.core.boundary_axis import BoundaryAxis

from codetocad.enums.axis import Axis
from codetocad.enums.length_unit import LengthUnit


class Dimension:
    COMPARISON_FLOAT_TOLERANCE = 0.001

    def __init__(self, value: float, unit: Optional[Union[str, LengthUnit]] = None):
        assert isinstance(value, (int, float)), "Dimension value must be a number."

        unit = (
            LengthUnit.from_string(unit.replace(" ", "").lower())
            if isinstance(unit, str)
            else unit
        )
        assert unit is None or isinstance(
            unit, LengthUnit
        ), "Dimension unit must be of type LengthUnit or None."

        self.value = value
        self.unit = unit

    @staticmethod
    def zero(unit: Optional[Union[str, LengthUnit]] = None) -> "Dimension":
        return Dimension(0, unit)

    def raise_power(self, power: float | int) -> "Dimension":
        return pow(self, power)

    @staticmethod
    def from_dimension_or_its_float_or_string_value(
        mystery_dimension: Union[str, float, "Dimension"],
        boundary_axis: Optional[BoundaryAxis] = None,
    ) -> "Dimension":
        if isinstance(mystery_dimension, Dimension):
            return mystery_dimension
        if isinstance(mystery_dimension, (int, float)):
            return Dimension(mystery_dimension)
        return Dimension.from_string(mystery_dimension, None, boundary_axis)

    def __str__(self) -> str:
        return f"{self.value}{' '+self.unit.name if self.unit else ''}"

    def __repr__(self) -> str:
        return self.__str__()

    def convert_to_unit(self, target_unit: Union[str, LengthUnit]) -> "Dimension":
        assert self.unit is not None, "Current dimension does not have a unit."
        target_unit = (
            LengthUnit.from_string(target_unit)
            if not isinstance(target_unit, LengthUnit)
            else target_unit
        )
        assert isinstance(
            target_unit, LengthUnit
        ), f"Could not convert to unit {target_unit}"

        newDimension = Dimension(
            self.value * (self.unit.value / target_unit.value), target_unit
        )
        return newDimension

    def arithmetic_precheck_and_unit_conversion(self, other) -> "Dimension":
        assert other is not None, "Right-hand value cannot be None."
        if not isinstance(other, Dimension):
            other = Dimension.from_string(other)
        if other.unit is not None and self.unit is not None and other.unit != self.unit:
            other = other.convert_to_unit(self.unit)
        if other.unit is None:
            other.unit = self.unit
        if self.unit is None:
            self.unit = other.unit
        return other

    def __add__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Dimension(self.value + other.value, self.unit or other.unit)

    def __sub__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Dimension(self.value - other.value, self.unit or other.unit)

    def __mul__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Dimension(self.value * other.value, self.unit or other.unit)

    def __truediv__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Dimension(self.value / other.value, self.unit or other.unit)

    def __floordiv__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Dimension(self.value // other.value, self.unit or other.unit)

    def __mod__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Dimension(self.value % other.value, self.unit or other.unit)

    # def __divmod__(self, other):
    #     other = self.arithmetic_precheck_and_unit_conversion(other)
    #     return Dimension(divmod(self.value, other.value), self.unit)

    def __pow__(self, other, mod=None):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Dimension(pow(self.value, other.value, mod), self.unit or other.unit)

    def __abs__(self):
        return Dimension(abs(self.value), self.unit)

    def __lt__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return self.value < other.value

    def __le__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return self.value <= other.value

    def __gt__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return self.value > other.value

    def __ge__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return self.value >= other.value

    def __eq__(self, other):
        if other is None:
            return False
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return int(self.value / self.COMPARISON_FLOAT_TOLERANCE) == int(
            other.value / self.COMPARISON_FLOAT_TOLERANCE
        )

    def __ne__(self, other):
        return not (self == other)

    def __copy__(self):
        return self.__deepcopy__()

    def __deepcopy__(self):
        return Dimension(self.value, self.unit)

    def copy(self):
        return self.__deepcopy__()

    # from_string: takes a string with a math operation and an optional unit of measurement
    # Default unit is None (scale factor) if it's not passed in
    # examples: "1m", "1.5ft", "3/8in", "1", "1-(3/4)cm"
    # boundary_axis is required if min,center,max are used

    @staticmethod
    def from_string(
        from_string: Union[str, float, "Dimension"],
        default_unit: Optional[LengthUnit] = None,
        boundary_axis: Optional[BoundaryAxis] = None,
    ):
        if isinstance(from_string, Dimension):
            return from_string.copy()

        unit = (
            LengthUnit.from_string(default_unit.replace(" ", "").lower())
            if isinstance(default_unit, str)
            else default_unit
        )
        assert (unit is None and default_unit is None) or isinstance(
            unit, LengthUnit
        ), "Could not parse default unit."

        if isinstance(from_string, (int, float)):
            return Dimension(from_string, unit)

        assert isinstance(from_string, str), "from_string must be a string."

        from_string = from_string.replace(" ", "").lower()

        value = from_string

        from codetocad.utilities import (
            get_unit_in_string,
            replace_min_max_center_with_respective_value,
        )

        # check if a unit is passed into from_string, e.g. "1-(3/4)cm" -> cm
        unitInString = get_unit_in_string(from_string)
        if unitInString:
            value = from_string[0 : -1 * len(unitInString)]
            unitInString = LengthUnit.from_string(unitInString)
            unit = unitInString or unit

        # if min,max,center is used, try to parse those words into their respective values.
        if Axis.is_axis_name_in_string(value):
            assert (
                boundary_axis is not None
            ), "min,max,center keywords used, but boundary_axis is not known."
            if unit is None:
                unit = boundary_axis.unit

            assert unit, "Could not determine the unit to convert the boundary axis."

            value = replace_min_max_center_with_respective_value(
                value, boundary_axis, unit
            )

        assert len(value) > 0, "Dimension value cannot be empty."

        # Make sure our value only contains math operations and numbers as a weak safety check before passing it to `eval`
        assert re.match(
            "[+\-*\/%\d\(\)]+", value
        ), f"Value {value} contains characters that are not allowed."

        value = eval(value)

        return Dimension(value, unit)



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.entity_interface import EntityInterface

from codetocad.interfaces.exportable_interface import ExportableInterface


class SceneInterface(metaclass=ABCMeta):
    """
    Scene, camera, lighting, rendering, animation, simulation and GUI related functionality.
    """

    @abstractmethod
    def __init__(self, name: "str| None" = None, description: "str| None" = None):

        self.name = name
        self.description = description

    @staticmethod
    def default() -> "SceneInterface":
        """
        Get a Scene instance for the current scene.
        """

        print("default is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def create(
        self,
    ) -> Self:
        """
        Creates a new scene.
        """

        print("create is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def delete(
        self,
    ) -> Self:
        """
        Deletes a scene.
        """

        print("delete is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def is_exists(
        self,
    ) -> "bool":
        """
        Check if the scene exists
        """

        print("is_exists is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def get_selected_entity(
        self,
    ) -> "EntityInterface":
        """
        Get the selected entity in the Scene.
        """

        print(
            "get_selected_entity is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def export(
        self,
        file_path: "str",
        entities: "list[str|ExportableInterface]",
        overwrite: "bool" = True,
        scale: "float" = 1.0,
    ) -> Self:
        """
        Export the entire scene or specific entities.
        """

        print("export is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def set_default_unit(self, unit: "str|LengthUnit") -> Self:
        """
        Set the document's default measurements system.
        """

        print(
            "set_default_unit is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def create_group(self, name: "str") -> Self:
        """
        Create a new group
        """

        print(
            "create_group is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def delete_group(self, name: "str", remove_children: "bool") -> Self:
        """
        Delete a new group
        """

        print(
            "delete_group is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def remove_from_group(self, entity_name: "str", group_name: "str") -> Self:
        """
        Removes an existing entity from a group
        """

        print(
            "remove_from_group is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def assign_to_group(
        self,
        entities: "list[str|EntityInterface]",
        group_name: "str",
        remove_from_other_groups: "bool| None" = True,
    ) -> Self:
        """
        Assigns an existing entity to a new group
        """

        print(
            "assign_to_group is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_visible(
        self, entities: "list[str|EntityInterface]", is_visible: "bool"
    ) -> Self:
        """
        Change the visibiltiy of the entity.
        """

        print(
            "set_visible is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_background_image(
        self,
        file_path: "str",
        location_x: "str|float|Dimension| None" = 0,
        location_y: "str|float|Dimension| None" = 0,
    ) -> Self:
        """
        Set the scene background image. This can be an image or an HDRI texture.
        """

        print(
            "set_background_image is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.scene_interface import SceneInterface


from codetocad.interfaces.entity_interface import EntityInterface

from codetocad.interfaces.exportable_interface import ExportableInterface


class Scene(
    SceneInterface,
):

    def __init__(self, name: "str| None" = None, description: "str| None" = None):

        self.name = name
        self.description = description

    @staticmethod
    def default() -> "SceneInterface":

        print(
            "default called",
        )

        return Scene()

    def create(
        self,
    ) -> Self:

        print(
            "create called",
        )

        return self

    def delete(
        self,
    ) -> Self:

        print(
            "delete called",
        )

        return self

    def is_exists(
        self,
    ) -> "bool":

        print(
            "is_exists called",
        )

        return True

    def get_selected_entity(
        self,
    ) -> "EntityInterface":

        print(
            "get_selected_entity called",
        )

        return __import__("codetocad").Part("an entity")

    def export(
        self,
        file_path: "str",
        entities: "list[str|ExportableInterface]",
        overwrite: "bool" = True,
        scale: "float" = 1.0,
    ) -> Self:

        print("export called", f": {file_path}, {entities}, {overwrite}, {scale}")

        return self

    def set_default_unit(self, unit: "str|LengthUnit") -> Self:

        print("set_default_unit called", f": {unit}")

        return self

    def create_group(self, name: "str") -> Self:

        print("create_group called", f": {name}")

        return self

    def delete_group(self, name: "str", remove_children: "bool") -> Self:

        print("delete_group called", f": {name}, {remove_children}")

        return self

    def remove_from_group(self, entity_name: "str", group_name: "str") -> Self:

        print("remove_from_group called", f": {entity_name}, {group_name}")

        return self

    def assign_to_group(
        self,
        entities: "list[str|EntityInterface]",
        group_name: "str",
        remove_from_other_groups: "bool| None" = True,
    ) -> Self:

        print(
            "assign_to_group called",
            f": {entities}, {group_name}, {remove_from_other_groups}",
        )

        return self

    def set_visible(
        self, entities: "list[str|EntityInterface]", is_visible: "bool"
    ) -> Self:

        print("set_visible called", f": {entities}, {is_visible}")

        return self

    def set_background_image(
        self,
        file_path: "str",
        location_x: "str|float|Dimension| None" = 0,
        location_y: "str|float|Dimension| None" = 0,
    ) -> Self:

        print(
            "set_background_image called", f": {file_path}, {location_x}, {location_y}"
        )

        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.vertex_interface import VertexInterface


from codetocad.interfaces.projectable_interface import ProjectableInterface

from codetocad.interfaces.entity_interface import EntityInterface


from providers.sample.entity import Entity


class Vertex(VertexInterface, Entity):

    def __init__(
        self,
        name: "str",
        location: "Point",
        description: "str| None" = None,
        native_instance=None,
        parent_entity: "str|EntityInterface| None" = None,
    ):

        self.name = name
        self.location = location
        self.description = description
        self.native_instance = native_instance
        self.parent_entity = parent_entity

    def get_control_points(
        self,
    ) -> "list[Point]":

        print(
            "get_control_points called",
        )

        return ["Point.from_list_of_float_or_string([0,0,0])"]

    def set_control_points(
        self, points: "list[str|list[str]|list[float]|list[Dimension]|Point]"
    ) -> Self:

        print("set_control_points called", f": {points}")

        return self

    def project(self, project_from: "ProjectableInterface") -> "ProjectableInterface":

        print("project called", f": {project_from}")

        return __import__("codetocad").Sketch("a projected sketch")



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.


from typing import Self


from codetocad.interfaces.light_interface import LightInterface


from providers.sample.entity import Entity


class Light(LightInterface, Entity):

    def __init__(
        self, name: "str", description: "str| None" = None, native_instance=None
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance

    def set_color(
        self, r_value: "int|float", g_value: "int|float", b_value: "int|float"
    ) -> Self:

        print("set_color called", f": {r_value}, {g_value}, {b_value}")

        return self

    def create_sun(self, energy_level: "float") -> Self:

        print("create_sun called", f": {energy_level}")

        return self

    def create_spot(self, energy_level: "float") -> Self:

        print("create_spot called", f": {energy_level}")

        return self

    def create_point(self, energy_level: "float") -> Self:

        print("create_point called", f": {energy_level}")

        return self

    def create_area(self, energy_level: "float") -> Self:

        print("create_area called", f": {energy_level}")

        return self



from enum import Enum
from typing import Union


class Axis(str, Enum):
    X = "x"
    Y = "y"
    Z = "z"

    MAX = "max"
    MIN = "min"
    CENTER = "center"

    @staticmethod
    def _max_min_center():
        return [Axis.MIN, Axis.MAX, Axis.CENTER]

    def _arithmetic_check(self, other):

        if not isinstance(other, (str, Axis)):
            raise TypeError("Axis name can only be concatenated with a string or Axis.")

        if self not in Axis._max_min_center():
            raise TypeError(f"Only {Axis._max_min_center()} can be concatenated.")

    def __add__(self, other):
        self._arithmetic_check(other)

        return self.name + (other if isinstance(other, str) else other.name)

    @staticmethod
    def is_axis_name_in_string(string_to_check: str) -> bool:
        """
        Used to check if min, max or center are in a string, for example "min + 2mm" -> returns True.
        """
        for word in [axis.name.lower() for axis in Axis._max_min_center()]:
            if word in string_to_check.lower():
                return True
        return False

    @staticmethod
    def from_string(axis: Union[str, float, "Axis"]):
        if isinstance(axis, Axis):
            return axis
        axis = str(axis).lower()
        if axis == "x" or axis == "0":
            return Axis.X
        if axis == "y" or axis == "1":
            return Axis.Y
        if axis == "z" or axis == "2":
            return Axis.Z
        if axis == "max":
            return Axis.MAX
        if axis == "min":
            return Axis.MIN
        if axis == "center":
            return Axis.CENTER

        assert False, f"Invalid axis {axis}"



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.vertex_interface import VertexInterface


from codetocad.interfaces.landmarkable_interface import LandmarkableInterface

from codetocad.interfaces.subdividable_interface import SubdividableInterface

from codetocad.interfaces.projectable_interface import ProjectableInterface

from codetocad.interfaces.mirrorable_interface import MirrorableInterface

from codetocad.interfaces.patternable_interface import PatternableInterface

from codetocad.interfaces.entity_interface import EntityInterface


class EdgeInterface(
    EntityInterface,
    MirrorableInterface,
    PatternableInterface,
    SubdividableInterface,
    ProjectableInterface,
    LandmarkableInterface,
    metaclass=ABCMeta,
):
    """
    A curve bounded by two Vertices.
    """

    @abstractmethod
    def __init__(
        self,
        name: "str",
        v1: "VertexInterface",
        v2: "VertexInterface",
        description: "str| None" = None,
        native_instance=None,
        parent_entity: "str|EntityInterface| None" = None,
    ):

        self.name = name
        self.v1 = v1
        self.v2 = v2
        self.description = description
        self.native_instance = native_instance
        self.parent_entity = parent_entity

    @abstractmethod
    def offset(self, distance: "str|float|Dimension") -> "EdgeInterface":
        """
        Clone and offset this edge a distance away from this one. This returns a new edge in the same sketch.
        """

        print("offset is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def fillet(self, other_edge: "EdgeInterface", amount: "str|float|Angle") -> Self:
        """
        Fillet this and another edge.
        """

        print("fillet is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def set_is_construction(self, is_construction: "bool") -> Self:
        """
        Mark this edge for construction only.
        """

        print(
            "set_is_construction is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_is_construction(
        self,
    ) -> "bool":
        """
        Check if this edge is for construction only.
        """

        print(
            "get_is_construction is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()
